@@ -1,8 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: bittop.scm,v 1.24 2003/02/14 18:28:00 cph Exp $
-
-Copyright (c) 1988-1999 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +19,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -156,7 +157,7 @@ USA.
 		       code-block)
 	  ((ucode-primitive primitive-object-set! 3)
 	   output-block 0
-	   (object-new-type (ucode-type null) total-length))
+	   (object-new-type (ucode-type manifest-vector) total-length))
 	  (insert-objects! output-block objects (fix:+ bl 1))
 	  output-block))))
 
