@@ -1,10 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: toplev.scm,v 4.75 2007/06/13 13:33:43 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -39,7 +38,8 @@ USA.
 (define compile-file)
 (let ((scm-pathname (lambda (path) (pathname-new-type path "scm")))
       (bin-pathname (lambda (path) (pathname-new-type path "bin")))
-      (ext-pathname (lambda (path) (pathname-new-type path "ext")))
+      (ext-pathname (lambda (path) (pathname-default-type path "ext")))
+      (ext-pathname? (lambda (path) (equal? (pathname-type path) "ext")))
       (com-pathname
        (lambda (path)
 	 (pathname-new-type path (compiler:compiled-code-pathname-type)))))
@@ -95,9 +95,11 @@ USA.
 			  (sf/default-declarations
 			   `((USUAL-INTEGRATIONS
 			      ,@compile-file:override-usual-integrations)
-			     ,@(if (null? dependencies)
-				   '()
-				   `((INTEGRATE-EXTERNAL ,@dependencies))))))
+			     ,@(let ((deps (keep-matching-items
+					    dependencies ext-pathname?)))
+				 (if (null? deps)
+				     '()
+				     `((INTEGRATE-EXTERNAL ,@deps)))))))
 		(sf input-file output-file))))
 	  (if (not compile-file:sf-only?)
 	      (process-file (bin-pathname file)
@@ -152,13 +154,17 @@ USA.
 		 (lambda (lap-output-port)
 		   (fluid-let ((*debugging-key*
 				(random-byte-vector 32)))
-		     (compile-scode/internal
-		      scode
-		      (pathname-new-type
-		       output-pathname
-		       (compiler:compiled-inf-pathname-type))
-		      rtl-output-port
-		      lap-output-port)))))))))))
+		     (compile-scode/file/hook
+		      input-pathname
+		      output-pathname
+		      (lambda ()
+			(compile-scode/internal
+			 scode
+			 (pathname-new-type
+			  output-pathname
+			  (compiler:compiled-inf-pathname-type))
+			 rtl-output-port
+			 lap-output-port)))))))))))))
   unspecific)
 
 (define *debugging-key*)
@@ -252,7 +258,7 @@ USA.
 (define (compile-scode/no-file scode keep-debugging-info?)
   (fluid-let ((compiler:noisy? #f)
 	      (*info-output-filename* keep-debugging-info?))
-    (compile-scode/internal/hook
+    (compile-scode/no-file/hook
      (lambda ()
        (compile-scode/internal scode keep-debugging-info?)))))
 
@@ -319,15 +325,17 @@ USA.
 	   (fluid-let ((*recursive-compilation-number* my-number)
 		       (compiler:package-optimization-level 'NONE)
 		       (*procedure-result?* procedure-result?))
-	     (compile-scode/internal
-	      scode
-	      (and *info-output-filename*
-		   (if (eq? *info-output-filename* 'KEEP)
-		       'KEEP
-		       'RECURSIVE))
-	      *rtl-output-port*
-	      *lap-output-port*
-	      bind-compiler-variables)))))
+	     (compile-scode/recursive/hook
+	      (lambda ()
+		(compile-scode/internal
+		 scode
+		 (and *info-output-filename*
+		      (if (eq? *info-output-filename* 'KEEP)
+			  'KEEP
+			  'RECURSIVE))
+		 *rtl-output-port*
+		 *lap-output-port*
+		 bind-compiler-variables)))))))
     (if procedure-result?
 	(let ((do-it
 	       (lambda ()
@@ -376,6 +384,11 @@ USA.
 (define *root-expression*)
 (define *root-procedure*)
 
+;; First set: phase/fg-generation
+;; Last used: [end]
+(define *tl-bound*)
+(define *tl-free*)
+
 ;; First set: phase/rtl-generation
 ;; Last used: phase/lap-linearization
 (define *rtl-expression*)
@@ -409,30 +422,38 @@ USA.
 	 (lambda ()
 	   (let ((value
 		  (let ((expression (thunk)))
-		    (let ((others
-			   (map (lambda (other) (vector-ref other 2))
-				(recursive-compilation-results))))
-		      (cond ((not (compiled-code-address? expression))
-			     (vector compiler:compile-by-procedures?
-				     expression
-				     others))
-			    ((null? others)
-			     expression)
-			    (else
-			     (scode/make-comment
-			      (make-dbg-info-vector
-			       (let ((all-blocks
-				      (list->vector
-				       (cons
-					(compiled-code-address->block
-					 expression)
-					others))))
-				 (if compiler:compile-by-procedures?
-				     (list 'COMPILED-BY-PROCEDURES
-					   all-blocks
-					   (list->vector others))
-				     all-blocks)))
-			      expression)))))))
+		    (let ((others (recursive-compilation-results)))
+		      (if (compiled-code-address? expression)
+			  (scode/make-comment
+			   (make-dbg-info-vector
+			    (if compiler:compile-by-procedures?
+				'COMPILED-BY-PROCEDURES
+				'COMPILED-AS-UNIT)
+			    (compiled-code-address->block expression)
+			    (list->vector
+			     (map (lambda (other)
+				    (vector-ref other 2))
+				  others))
+			    (list->vector
+			     (apply lset-union
+				    equal?
+				    *tl-bound*
+				    (map (lambda (other)
+					   (vector-ref other 3))
+					 others)))
+			    (list->vector
+			     (apply lset-union
+				    equal?
+				    *tl-free*
+				    (map (lambda (other)
+					   (vector-ref other 4))
+					 others))))
+			   expression)
+			  (vector compiler:compile-by-procedures?
+				  expression
+				  (map (lambda (other)
+					 (vector-ref other 2))
+				       others)))))))
 	     (if compiler:show-time-reports?
 		 (compiler-time-report "Total compilation time"
 				       *process-time*
@@ -472,6 +493,8 @@ USA.
 		(*root-expression*)
 		(*root-procedure*)
 		(*root-block*)
+		(*tl-bound*)
+		(*tl-free*)
 		(*rtl-expression*)
 		(*rtl-procedures*)
 		(*rtl-continuations*)
@@ -509,6 +532,8 @@ USA.
   (set! *root-expression*)
   (set! *root-procedure*)
   (set! *root-block*)
+  (set! *tl-bound*)
+  (set! *tl-free*)
   (set! *rtl-expression*)
   (set! *rtl-procedures*)
   (set! *rtl-continuations*)
@@ -564,11 +589,11 @@ USA.
 	 (phase/rtl-generation)
 	 (phase/rtl-optimization)
 	 (if rtl-output-port
-	     (phase/rtl-file-output rtl-output-port))
+	     (phase/rtl-file-output scode rtl-output-port))
 	 (phase/lap-generation)
 	 (phase/lap-linearization)
 	 (if lap-output-port
-	     (phase/lap-file-output lap-output-port))
+	     (phase/lap-file-output scode lap-output-port))
 	 (if compiler:cross-compiling?
 	     (cross-assemble&link info-output-pathname)
 	     (assemble&link info-output-pathname)))))))
@@ -643,8 +668,11 @@ USA.
 (define (phase/canonicalize-scode)
   (compiler-subphase "Scode Canonicalization"
     (lambda ()
-      (set! *scode* (canonicalize/top-level (last-reference *input-scode*)))
-      unspecific)))
+      (receive (scode bound)
+	  (canonicalize/top-level (last-reference *input-scode*))
+	(set! *scode* scode)
+	(set! *tl-bound* bound)
+	unspecific))))
 
 (define (phase/translate-scode)
   (compiler-subphase "Translation of Scode into Flow Graph"
@@ -668,6 +696,11 @@ USA.
 		  (error "Value of procedure compilation not procedure" node))
 	      (set! *root-procedure* operand))))
       (set! *root-block* (expression-block *root-expression*))
+      (if (not *tl-bound*)
+	  (set! *tl-bound*
+		(map variable-name (block-bound-variables *root-block*))))
+      (set! *tl-free*
+	    (map variable-name (block-free-variables *root-block*)))
       (if (or (null? *expressions*)
 	      (not (null? (cdr *expressions*))))
 	  (error "Multiple expressions"))
