@@ -1,8 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: closan.scm,v 4.32 2003/02/14 18:28:01 cph Exp $
-
-Copyright (c) 1987-1991, 1998, 1999, 2001, 2002 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +19,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -303,15 +304,18 @@ USA.
   ;; if it was an ancestor before procedure-drifting took place, don't
   ;; close, just undrift.
   (let ((procedure (condition-procedure condition)))
-    (cond ((block-ancestor-or-self? block block*)
-	   unspecific)
-	  ((and (original-block-ancestor? block block*)
-		(not (procedure-closure-context procedure)))
-	   (undrifting-constraint! block block* condition))
-	  (else
-	   (close-procedure! procedure
-			     (condition-keyword condition)
-			     (condition-argument condition))))))
+    (define (close)
+      (close-procedure! procedure
+			(condition-keyword condition)
+			(condition-argument condition)))
+    (cond ((and (ic-block? block*)
+		(not (eq? block block*)))
+	   (close))
+	  ((not (block-ancestor-or-self? block block*))
+	   (if (and (original-block-ancestor? block block*)
+		    (not (procedure-closure-context procedure)))
+	       (undrifting-constraint! block block* condition)
+	       (close))))))
 
 (define (close-procedure! procedure keyword argument)
   (add-closure-reason! procedure keyword argument)
