@@ -1,8 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: lapgen.scm,v 1.17 2004/07/01 01:19:57 cph Exp $
-
-Copyright 1993,1998,2001,2002,2004 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +19,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -85,7 +86,7 @@ USA.
 	(else
 	 (comp-internal-error "Unknown machine register"
 			      'MACHINE-REGISTER-NAME reg))))
-    
+
 (define (machine-register-type reg)
   (cond ((eq? reg regnum:value)
 	 "SCHEME_OBJECT")
@@ -102,10 +103,10 @@ USA.
   (< reg number-of-machine-registers))
 
 (define (rhs-cast reg type)
-  (string-append "((" (type->name type) ") " reg ")"))
+  (c:cast (type->name type) reg))
 
 (define (lhs-cast reg type)
-  (string-append "(* ((" (type->name type) " *) &" reg "))"))
+  (c:* (c:cast (type->name type) (c:& reg))))
 
 (define permanent-register-list)
 (define current-register-list)
@@ -210,12 +211,13 @@ USA.
 	   name))))
 
 (define (register-declarations)
-  (append-map
-   (lambda (register)
-     (map (lambda (spec)
-	    (string-append "\t" (type->name (car spec)) " " (cdr spec) ";\n"))
-	  (cdr register)))
-   permanent-register-list))
+  (c:group*
+   (append-map
+    (lambda (register)
+      (map (lambda (spec)
+	     (c:decl (type->name (car spec)) (cdr spec)))
+	   (cdr register)))
+    permanent-register-list)))
 
 (define (standard-move-to-target! src tgt)
   (let ((src-type (register-type src)))
@@ -226,11 +228,11 @@ USA.
 	  ((register-is-machine-register? tgt)
 	   (let ((src (standard-source! src
 					(machine-register-type-symbol tgt))))
-	     (LAP ,(machine-register-name tgt) " = " ,src ";\n\t")))
+	     (LAP ,(c:= (machine-register-name tgt) src))))
 	  ((register-is-machine-register? src)
 	   (let ((tgt (standard-target! tgt
 					(machine-register-type-symbol src))))
-	     (LAP ,tgt " = " ,(machine-register-name src) ";\n\t")))
+	     (LAP ,(c:= tgt (machine-register-name src)))))
 	  (else
 	   (let ((reg-type
 		  (case src-type
@@ -240,35 +242,109 @@ USA.
 		     (comp-internal-error "Unknown RTL register type"
 					  'STANDARD-MOVE-TO-TARGET!
 					  src-type)))))
-	     (LAP ,(find-register! tgt reg-type) " = "
-		  ,(find-register! src reg-type) ";\n\t"))))))
+	     (LAP ,(c:= (find-register! tgt reg-type)
+			(find-register! src reg-type))))))))
 
-;;;; Communicate with cout.scm
+;;;; Communicate with "cout.scm"
+
+(define (c:invoke-interface-0 code)
+  (use-invoke-interface! 0)
+  (c:scall "INVOKE_INTERFACE_0" code))
+
+(define (c:invoke-interface-1 code arg1)
+  (use-invoke-interface! 1)
+  (c:scall "INVOKE_INTERFACE_1" code arg1))
+
+(define (c:invoke-interface-2 code arg1 arg2)
+  (use-invoke-interface! 2)
+  (c:scall "INVOKE_INTERFACE_2" code arg1 arg2))
+
+(define (c:invoke-interface-3 code arg1 arg2 arg3)
+  (use-invoke-interface! 3)
+  (c:scall "INVOKE_INTERFACE_3" code arg1 arg2 arg3))
+
+(define (c:invoke-interface-4 code arg1 arg2 arg3 arg4)
+  (use-invoke-interface! 4)
+  (c:scall "INVOKE_INTERFACE_4" code arg1 arg2 arg3 arg4))
 
 (define (use-invoke-interface! number)
   (set! *invoke-interface*
 	(let ((old *invoke-interface*))
-	  (if (eq? old 'infinity)
+	  (if (eq? old 'INFINITY)
 	      number
-	      (min old number)))))
+	      (min old number))))
+  unspecific)
 
-(define (use-invoke-primitive!)
-  (set! *used-invoke-primitive* true))
+(define (c:invoke-primitive prim arity)
+  (set! *used-invoke-primitive* #t)
+  (c:scall "INVOKE_PRIMITIVE" prim arity))
 
-(define (use-closure-interrupt-check!)
-  (use-invoke-interface! 0))
+(define (c:closure-interrupt-check)
+  (use-invoke-interface! 0)
+  (c:scall "CLOSURE_INTERRUPT_CHECK" code:compiler-interrupt-closure))
 
-(define (use-interrupt-check!)
-  (use-invoke-interface! 1))
+(define (c:interrupt-check code block-label)
+  (use-invoke-interface! 1)
+  (c:scall "INTERRUPT_CHECK" code block-label))
 
-(define (use-dlink-interrupt-check!)
-  (use-invoke-interface! 2))
+(define (c:dlink-interrupt-check block-label)
+  (use-invoke-interface! 2)
+  (c:scall "DLINK_INTERRUPT_CHECK"
+	   code:compiler-interrupt-dlink
+	   block-label))
 
-(define (use-jump-execute-chache!)
-  (set! *use-jump-execute-chache* #t))
+(define (c:jump address)
+  (set! *use-jump-execute-chache* #t)
+  (c:scall "JUMP" address))
 
-(define (use-pop-return!)
-  (set! *use-pop-return* #t))
+(define (c:pop-return)
+  (set! *use-pop-return* #t)
+  (c:goto 'pop_return))
+
+(define (c:reg-block)
+  "Rrb")
+
+(define (c:free-reg)
+  "Rhp")
+
+(define (c:sp-reg)
+  "Rsp")
+
+(define (c:val-reg)
+  "Rvl")
+
+(define (c:dlink-reg)
+  "Rdl")
+
+(define (c:pc-reg)
+  "Rpc")
+
+(define (c:rref index)
+  (c:aref (c:reg-block) index))
+
+(define (c:env-reg)
+  (c:rref "REGBLOCK_ENV"))
+
+(define (c:push object)
+  (c:= (c:* (c:predec (c:sp-reg))) object))
+
+(define (c:pop)
+  (c:* (c:postinc (c:sp-reg))))
+
+(define (c:tos)
+  (c:* (c:sp-reg)))
+
+(define (c:sref index)
+  (c:aref (c:sp-reg) index))
+
+(define (c:sptr index)
+  (c:aptr (c:sp-reg) index))
+
+(define (c:cref index)
+  (c:aref 'current_block index))
+
+(define (c:cptr index)
+  (c:aptr 'current_block index))
 
 ;;;; Constants, Labels, and Various Caches
 
@@ -548,23 +624,23 @@ USA.
 		       'REGISTER->HOME-TRANSFER one two))
 
 (define (lap:make-label-statement label)
-  (LAP "\n" ,label ":\n\t" ))
+  (LAP ,(c:label label)))
 
 (define (lap:make-unconditional-branch label)
-  (LAP "goto " ,label ";\n\t"))
+  (LAP ,(c:goto label)))
 
 (define (lap:make-entry-point label block-start-label)
   block-start-label			; ignored
   (declare-block-label! expression-code-word label #f)
   (lap:make-label-statement label))
 
-(define (compare cc val1 val2)
-  (set-current-branches!
-   (lambda (label)
-     (LAP "if (" ,val1 ,cc ,val2 ")\n\t  goto " ,label ";\n\t"))
-   (lambda (label)
-     (LAP "if (!(" ,val1 ,cc ,val2 "))\n\t  goto " ,label ";\n\t")))
+(define (branch-on-expr expr)
+  (set-current-branches! (lambda (label) (LAP ,(c:if-goto expr label)))
+			 (lambda (label) (LAP ,(c:if-goto (c:! expr) label))))
   (LAP))
+
+(define (compare c:?? val1 val2)
+  (branch-on-expr (c:?? val1 val2)))
 
 (define (define-arithmetic-method operator methods method)
   (let ((entry (assq operator (cdr methods))))
@@ -578,32 +654,33 @@ USA.
 	   (comp-internal-error "Unknown operator" 'LOOKUP-ARITHMETIC-METHOD
 				operator))))
 
-(let-syntax ((define-codes
-	       (sc-macro-transformer
-		(lambda (form environment)
-		  environment
-		  `(BEGIN
-		     ,@(let loop ((names (cddr form)) (index (cadr form)))
-			 (if (pair? names)
-			     (cons `(DEFINE-INTEGRABLE
-				      ,(symbol-append 'CODE:COMPILER-
-						      (car names))
-				      ,index)
-				   (loop (cdr names) (1+ index)))
-			     `())))))))
-  (define-codes #x012
-    primitive-apply primitive-lexpr-apply
-    apply error lexpr-apply link
-    interrupt-closure interrupt-dlink interrupt-procedure 
-    interrupt-continuation interrupt-ic-procedure
-    assignment-trap cache-reference-apply
-    reference-trap safe-reference-trap unassigned?-trap
-    -1+ &/ &= &> 1+ &< &- &* negative? &+ positive? zero?
-    access lookup safe-lookup unassigned? unbound?
-    set! define lookup-apply primitive-error
-    quotient remainder modulo
-    reflect-to-interface interrupt-continuation-2
-    compiled-code-bkpt compiled-closure-bkpt))
+(define-syntax define-codes
+  (sc-macro-transformer
+   (lambda (form environment)
+     environment
+     `(BEGIN
+	,@(let loop ((names (cddr form)) (index (cadr form)))
+	    (if (pair? names)
+		(cons `(DEFINE-INTEGRABLE
+			 ,(symbol-append 'CODE:COMPILER-
+					 (car names))
+			 ,index)
+		      (loop (cdr names) (1+ index)))
+		`()))))))
+
+(define-codes #x012
+  primitive-apply primitive-lexpr-apply
+  apply error lexpr-apply link
+  interrupt-closure interrupt-dlink interrupt-procedure
+  interrupt-continuation interrupt-ic-procedure
+  assignment-trap cache-reference-apply
+  reference-trap safe-reference-trap unassigned?-trap
+  -1+ &/ &= &> 1+ &< &- &* negative? &+ positive? zero?
+  access lookup safe-lookup unassigned? unbound?
+  set! define lookup-apply primitive-error
+  quotient remainder modulo
+  reflect-to-interface interrupt-continuation-2
+  compiled-code-bkpt compiled-closure-bkpt)
 
 
 (define (pre-lapgen-analysis rgraphs)
