@@ -1,8 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: lapgen.scm,v 1.34 2003/02/14 18:28:03 cph Exp $
-
-Copyright 1992,1993,1998,2001,2002,2003 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +19,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -66,9 +67,6 @@ USA.
 	(else
 	 (error "unable to determine register type" register))))
 
-(define (register-types-compatible? type1 type2)
-  (boolean=? (eq? type1 'FLOAT) (eq? type2 'FLOAT)))
-
 (define register-reference
   (let ((references (make-vector number-of-machine-registers)))
     (let loop ((i 0))
@@ -136,8 +134,7 @@ USA.
 ;;;; Utilities for the register allocator interface
 
 (define-integrable (machine->machine-register source target)
-  (if (not (register-types-compatible? source target))
-      (error "Moving between incompatible register types" source target))
+  (guarantee-registers-compatible source target)
   (if (not (float-register? source))
       (LAP (MOV W ,(register-reference target) ,(register-reference source)))
       (let ((ssti (floreg->sti source))
@@ -676,7 +673,9 @@ USA.
   shortcircuit-apply-size-7
   shortcircuit-apply-size-8
   interrupt-continuation-2
-  conditionally-serialize)
+  conditionally-serialize
+  fixnum-shift
+  set-interrupt-enables!)
 
 ;; Operation tables
 
