@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -405,7 +406,6 @@ If you want VALUE to be a string, you must surround it with doublequotes."
 	    (find-escape next comtabs)))
 
     (define (show-bindings argument next comtabs)
-      comtabs
       (cons (call-with-output-string
 	     (lambda (port)
 	       (describe-bindings (mode-comtabs (name->mode argument 'ERROR))
