@@ -1,8 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: macros.scm,v 1.75 2003/02/14 18:28:12 cph Exp $
-
-Copyright 1986, 1989-1999, 2001, 2002 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +19,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -29,6 +30,31 @@ USA.
 
 ;; Upwards compatibility:
 (define edwin-syntax-table (->environment '(EDWIN)))
+
+(define-syntax define-editor-alias
+  (sc-macro-transformer
+   (lambda (form env)
+     env
+     (if (syntax-match? '(SYMBOL SYMBOL SYMBOL) (cdr form))
+	 (let ((type (cadr form))
+	       (new (caddr form))
+	       (old (cadddr form)))
+	   (receive (table name-map)
+	       (case type
+		 ((MODE)
+		  (values 'EDITOR-MODES mode-name->scheme-name))
+		 ((COMMAND)
+		  (values 'EDITOR-COMMANDS command-name->scheme-name))
+		 ((VARIABLE)
+		  (values 'EDITOR-VARIABLES variable-name->scheme-name))
+		 (else
+		  (error "Unknown alias type:" type)))
+	   `(BEGIN
+	      (DEFINE ,(name-map new) ,(name-map old))
+	      (STRING-TABLE-PUT! ,table
+				 ,(symbol->string new)
+				 ,(name-map old)))))
+	 (ill-formed-syntax form)))))
 
 (define-syntax define-command
   (rsc-macro-transformer
@@ -105,7 +131,7 @@ USA.
 	,(variable-name->scheme-name (list-ref form 1))
 	(,(close-syntax 'MAKE-VARIABLE environment)
 	 ',(list-ref form 1)
-	 ,(list-ref form 2)
+	 ,(if (> (length form) 2) (list-ref form 2) '#F)
 	 ,(if (> (length form) 3) (list-ref form 3) '#F)
 	 ,buffer-local?
 	 ,(if (> (length form) 4) (list-ref form 4) '#F)
