@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -37,7 +38,19 @@ USA.
   (with-working-directory-pathname "sos"
     (lambda ()
       (load "load")))
-  (for-each compile-dir '("xml" "win32" "edwin" "imail" "ssp")))
+  (for-each compile-dir '("xml" "win32" "edwin" "imail" "ssp" "ffi")))
+
+(define (compile-ffi dir)
+  (if (eq? microcode-id/compiled-code-type 'C)
+      (in-liarc
+       (lambda ()
+	 (c-compile-dir dir)
+	 (let* ((line '("make" "compile-liarc-bundle"))
+		(code (run-synchronous-subprocess
+		       (car line) (cdr line) 'working-directory dir)))
+	   (if (not (zero? code))
+	       (error "Process exited with error code:" code line)))))
+      (compile-dir dir)))
 
 (define (compile-boot-dirs compile-dir)
   (compile-cref compile-dir)
