@@ -2,8 +2,8 @@
 #
 # Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
 #     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-#     2005, 2006, 2007, 2008, 2009, 2010 Massachusetts Institute of
-#     Technology
+#     2005, 2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute
+#     of Technology
 #
 # This file is part of MIT/GNU Scheme.
 #
@@ -26,7 +26,7 @@ set -e
 
 . etc/functions.sh
 
-run_cmd "${@}" --heap 6000 --stack 200 <<EOF
+run_cmd "${@}" --heap 6000 --stack 200 --batch-mode <<EOF
 (begin
   (load "etc/compile.scm")
   (compile-everything))
