@@ -1,8 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: ed-ffi.scm,v 1.22 2003/02/14 18:28:14 cph Exp $
-
-Copyright 2000,2001,2003 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +19,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -30,6 +31,7 @@ USA.
     ("imail-core"	(edwin imail))
     ("imail-file"	(edwin imail file-folder))
     ("imail-imap"	(edwin imail imap-folder))
+    ("imail-mime"	(edwin imail))
     ("imail-rmail"	(edwin imail file-folder rmail-folder))
     ("imail-summary"	(edwin imail front-end summary))
     ("imail-top"	(edwin imail front-end))
