@@ -1,10 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: imail-umail.scm,v 1.57 2008/01/30 20:02:10 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -74,7 +73,7 @@ USA.
   folder
   (make-umail-message (message-header-fields message)
 		      (file-message-body message)
-		      (list-copy (message-flags message))
+		      (list-copy (message-permanent-flags message))
 		      (umail-message-from-line message)))
 
 (define-method message-internal-time ((message <umail-message>))
@@ -133,15 +132,11 @@ USA.
 		   (loop)))))))))
 
 (define (read-umail-message-1 folder from-line headers body)
-  (call-with-values
-      (lambda () (file-folder-strip-internal-headers folder headers))
-    (lambda (headers internal-headers)
-      (call-with-values
-	  (lambda ()
-	    (parse-imail-header-fields internal-headers))
-	(lambda (internal-headers flags)
-	  internal-headers
-	  (make-umail-message headers body flags from-line))))))
+  (make-umail-message headers
+		      body
+		      (header-fields->message-flags
+		       (file-folder-internal-headers folder headers))
+		      from-line))
 
 (define (umail-delimiter? line)
   (re-string-match unix-mail-delimiter line))
@@ -153,7 +148,8 @@ USA.
     (lambda (port)
       (for-each-vector-element (file-folder-messages folder)
 	(lambda (message)
-	  (write-umail-message message #t port))))))
+	  (write-umail-message message #t port)))
+      (output-port/synchronize-output port))))
 
 (define-method append-message-to-file (message url (type <umail-folder-type>))
   type
@@ -164,10 +160,15 @@ USA.
 (define (write-umail-message message output-flags? port)
   (write-string (umail-message-from-line message) port)
   (newline port)
-  (if output-flags?
-      (write-header-field (message-flags->header-field (message-flags message))
-			  port))
-  (write-header-fields (message-header-fields message) port)
+  (write-header-fields
+   (if output-flags?
+       (append (message-header-fields message)
+	       (let ((flags (message-permanent-flags message)))
+		 (if (pair? flags)
+		     (list (message-flags->header-field flags))
+		     '())))
+       (message-header-fields message))
+   port)
   (for-each (lambda (line)
 	      (if (string-prefix-ci? "From " line)
 		  (write-string ">" port))
