@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -596,6 +597,15 @@ USA.
 (define (remove-property! object key)
   (set-object-properties! object (del-assq! key (object-properties object))))
 
+(define (intern-property! object key generator)
+  (let* ((default (cons 0 0))
+	 (datum (get-property object key default)))
+    (if (eq? datum default)
+	(let ((datum (generator object)))
+	  (store-property! object key datum)
+	  datum)
+	datum)))
+
 ;;;; Modification events
 
 (define-class <modification-event-mixin> ()
