@@ -1,8 +1,9 @@
 /* -*-C-*-
 
-$Id: daemon.c,v 9.33 2003/02/14 18:28:18 cph Exp $
-
-Copyright (c) 1987-1999 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +19,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 */
@@ -52,14 +53,14 @@ DEFINE_PRIMITIVE ("CLOSE-LOST-OPEN-FILES", Prim_close_lost_open_files, 1, 1, 0)
     SCHEME_OBJECT file_list = (ARG_REF (1));
     SCHEME_OBJECT * smash = (PAIR_CDR_LOC (file_list));
     SCHEME_OBJECT cell = (*smash);
-    while (cell != EMPTY_LIST)
+    while (!EMPTY_LIST_P (cell))
       {
-	SCHEME_OBJECT weak_cell = (FAST_PAIR_CAR (cell));
-	if ((FAST_PAIR_CAR (weak_cell)) == SHARP_F)
+	SCHEME_OBJECT weak_cell = (PAIR_CAR (cell));
+	if ((PAIR_CAR (weak_cell)) == SHARP_F)
 	  {
 	    OS_channel_close_noerror
-	      (UNSIGNED_FIXNUM_TO_LONG (FAST_PAIR_CDR (weak_cell)));
-	    cell = (FAST_PAIR_CDR (cell));
+	      (UNSIGNED_FIXNUM_TO_LONG (PAIR_CDR (weak_cell)));
+	    cell = (PAIR_CDR (cell));
 	    (*smash) = cell;
 	  }
 	else
@@ -81,62 +82,56 @@ DEFINE_PRIMITIVE ("CLOSE-LOST-OPEN-FILES", Prim_close_lost_open_files, 1, 1, 0)
    Therefore, there is no gc check here. */
 
 static void
-DEFUN (rehash_pair, (pair, hash_table, table_size),
-       SCHEME_OBJECT pair AND SCHEME_OBJECT hash_table
-       AND long table_size)
+rehash_pair (SCHEME_OBJECT pair, SCHEME_OBJECT hash_table,
+       long table_size)
 {
   long object_datum, hash_address;
   SCHEME_OBJECT * new_pair;
 
-  object_datum = (OBJECT_DATUM (FAST_PAIR_CAR (pair)));
+  object_datum = (OBJECT_DATUM (PAIR_CAR (pair)));
   hash_address = (2 + (object_datum % table_size));
   new_pair = Free;
   *Free++ = (OBJECT_NEW_TYPE (TC_LIST, pair));
-  *Free++ = (FAST_MEMORY_REF (hash_table, hash_address));
-  FAST_MEMORY_SET (hash_table,
+  *Free++ = (MEMORY_REF (hash_table, hash_address));
+  MEMORY_SET (hash_table,
 		   hash_address,
 		   (MAKE_POINTER_OBJECT (TC_LIST, new_pair)));
-  return;
 }
 
 static void
-DEFUN (rehash_bucket, (bucket, hash_table, table_size),
-       SCHEME_OBJECT * bucket AND SCHEME_OBJECT hash_table
-       AND long table_size)
+rehash_bucket (SCHEME_OBJECT * bucket, SCHEME_OBJECT hash_table,
+       long table_size)
 {
-  fast SCHEME_OBJECT weak_pair;
+  SCHEME_OBJECT weak_pair;
 
-  while (*bucket != EMPTY_LIST)
+  while (!EMPTY_LIST_P (*bucket))
   {
-    weak_pair = (FAST_PAIR_CAR (*bucket));
-    if ((FAST_PAIR_CAR (weak_pair)) != SHARP_F)
+    weak_pair = (PAIR_CAR (*bucket));
+    if ((PAIR_CAR (weak_pair)) != SHARP_F)
     {
       rehash_pair (weak_pair, hash_table, table_size);
     }
     bucket = (PAIR_CDR_LOC (*bucket));
   }
-  return;
 }
 
 static void
-DEFUN (splice_and_rehash_bucket, (bucket, hash_table, table_size),
-       SCHEME_OBJECT * bucket AND SCHEME_OBJECT hash_table
-       AND long table_size)
+splice_and_rehash_bucket (SCHEME_OBJECT * bucket, SCHEME_OBJECT hash_table,
+       long table_size)
 {
-  fast SCHEME_OBJECT weak_pair;
+  SCHEME_OBJECT weak_pair;
 
-  while ((*bucket) != EMPTY_LIST)
+  while (!EMPTY_LIST_P (*bucket))
   {
-    weak_pair = (FAST_PAIR_CAR (*bucket));
-    if ((FAST_PAIR_CAR (weak_pair)) != SHARP_F)
+    weak_pair = (PAIR_CAR (*bucket));
+    if ((PAIR_CAR (weak_pair)) != SHARP_F)
     {
       rehash_pair (weak_pair, hash_table, table_size);
       bucket = (PAIR_CDR_LOC (*bucket));
     }
     else
-      *bucket = (FAST_PAIR_CDR (*bucket));
+      *bucket = (PAIR_CDR (*bucket));
   }
-  return;
 }
 
 /* (REHASH unhash-table hash-table)
@@ -163,7 +158,7 @@ DEFINE_PRIMITIVE ("REHASH", Prim_rehash, 2, 2, 0)
   bucket = (MEMORY_LOC ((ARG_REF (1)), 1));
   while ((counter--) > 0)
     {
-      if ((FAST_PAIR_CAR (*bucket)) == SHARP_T)
+      if ((PAIR_CAR (*bucket)) == SHARP_T)
 	splice_and_rehash_bucket
 	  ((PAIR_CDR_LOC (*bucket)), (ARG_REF (2)), table_size);
       else
