@@ -1,23 +1,27 @@
 /* -*-C-*-
 
-$Id: lookup.c,v 9.68 2001/12/21 18:18:21 cph Exp $
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
-Copyright (c) 1988-2001 Massachusetts Institute of Technology
+This file is part of MIT/GNU Scheme.
 
-This program is free software; you can redistribute it and/or modify
+MIT/GNU Scheme is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or (at
 your option) any later version.
 
-This program is distributed in the hope that it will be useful, but
+MIT/GNU Scheme is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+along with MIT/GNU Scheme; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
+
 */
 
 /* Environment lookup, modification, and definition.  */
@@ -25,21 +29,6 @@ USA.
 #include "scheme.h"
 #include "trap.h"
 #include "lookup.h"
-
-extern long make_uuo_link
-  (SCHEME_OBJECT, SCHEME_OBJECT, SCHEME_OBJECT, unsigned long);
-extern long make_fake_uuo_link
-  (SCHEME_OBJECT, SCHEME_OBJECT, unsigned long);
-extern SCHEME_OBJECT extract_uuo_link
-  (SCHEME_OBJECT, unsigned long);
-
-extern SCHEME_OBJECT extract_variable_cache
-  (SCHEME_OBJECT, unsigned long);
-extern void store_variable_cache
-  (SCHEME_OBJECT, SCHEME_OBJECT, unsigned long);
-
-extern SCHEME_OBJECT compiled_block_environment
-  (SCHEME_OBJECT);
 
 /* Hopefully a conservative guesstimate. */
 #ifndef SPACE_PER_UUO_LINK	/* So it can be overriden from config.h */
@@ -71,9 +60,9 @@ extern SCHEME_OBJECT compiled_block_environment
 
 #define GC_CHECK(n)							\
 {									\
-  if (GC_Check (n))							\
+  if (GC_NEEDED_P (n))							\
     {									\
-      Request_GC (n);							\
+      REQUEST_GC (n);							\
       return (PRIM_INTERRUPT);						\
     }									\
 }
@@ -88,31 +77,55 @@ extern SCHEME_OBJECT compiled_block_environment
    ? EXTERNAL_UNASSIGNED_OBJECT						\
    : (value))
 
-#define EXTERNAL_UNASSIGNED_OBJECT (Get_Fixed_Obj_Slot (Non_Object))
+#define EXTERNAL_UNASSIGNED_OBJECT					\
+  (VECTOR_REF (fixed_objects, NON_OBJECT))
+
+#define PALIST_COND(palist_var) (PAIR_P (*palist_var))
+
+#define PALIST_HEADER(palist_var, prefs_var)				\
+  SCHEME_OBJECT * prefs_var = (PAIR_CDR_LOC (PAIR_CAR (*palist_var)));
+
+#define PALIST_FOOTER(palist_var) do					\
+{									\
+  if (PAIR_P (PAIR_CDR (PAIR_CAR (*palist_var))))			\
+    palist_var = (PAIR_CDR_LOC (*palist_var));				\
+  else									\
+    (*palist_var) = (PAIR_CDR (*palist_var));				\
+} while (false)
+
+#define PREFS_COND(prefs_var) (PAIR_P (*prefs_var))
+
+#define PREFS_HEADER(prefs_var)						\
+  PREFS_HEADER_1 (prefs_var, (PAIR_CAR (*prefs_var)))
+
+#define PREFS_HEADER_1(prefs_var, cache)				\
+{									\
+  if ((GET_CACHE_REFERENCE_BLOCK (cache)) == SHARP_F)			\
+    {									\
+      (*prefs_var) = (PAIR_CDR (*prefs_var));				\
+      continue;								\
+    }									\
+}
+
+#define PREFS_FOOTER(prefs_var) do					\
+{									\
+  prefs_var = (PAIR_CDR_LOC (*prefs_var));				\
+} while (false)
 
 #define WALK_REFERENCES(refs_pointer, ref_var, body)			\
 {									\
   SCHEME_OBJECT * WR_palist = (refs_pointer);				\
-  while (PAIR_P (*WR_palist))						\
+  while (PALIST_COND (WR_palist))					\
     {									\
-      SCHEME_OBJECT * WR_prefs						\
-	= (PAIR_CDR_LOC (PAIR_CAR (*WR_palist)));			\
-      while (PAIR_P (*WR_prefs))					\
+      PALIST_HEADER (WR_palist, WR_prefs);				\
+      while (PREFS_COND (WR_prefs))					\
 	{								\
 	  SCHEME_OBJECT ref_var = (PAIR_CAR (*WR_prefs));		\
-	  if ((GET_CACHE_REFERENCE_BLOCK (ref_var))			\
-	      == SHARP_F)						\
-	    (*WR_prefs) = (PAIR_CDR (*WR_prefs));			\
-	  else								\
-	    {								\
-	      body;							\
-	      WR_prefs = (PAIR_CDR_LOC (*WR_prefs));			\
-	    }								\
+	  PREFS_HEADER_1 (WR_prefs, ref_var);				\
+	  body;								\
+	  PREFS_FOOTER (WR_prefs);					\
 	}								\
-      if (PAIR_P (PAIR_CDR (PAIR_CAR (*WR_palist))))			\
-	WR_palist = (PAIR_CDR_LOC (*WR_palist));			\
-      else								\
-	(*WR_palist) = (PAIR_CDR (*WR_palist));				\
+      PALIST_FOOTER (WR_palist);					\
     }									\
 }
 
@@ -124,36 +137,20 @@ static long assign_variable_end
   (SCHEME_OBJECT *, SCHEME_OBJECT, SCHEME_OBJECT *, int);
 static long assign_variable_cache
   (SCHEME_OBJECT, SCHEME_OBJECT, SCHEME_OBJECT *, int);
-static long update_uuo_links
-  (SCHEME_OBJECT, SCHEME_OBJECT);
 static long guarantee_extension_space
   (SCHEME_OBJECT);
 static long allocate_frame_extension
   (unsigned long, SCHEME_OBJECT, SCHEME_OBJECT *);
-static void move_all_references
-  (SCHEME_OBJECT, SCHEME_OBJECT, unsigned int);
 static long unbind_cached_variable
   (SCHEME_OBJECT *, SCHEME_OBJECT, SCHEME_OBJECT);
 static void unbind_variable_1
   (SCHEME_OBJECT *, SCHEME_OBJECT, SCHEME_OBJECT);
-static long add_cache_reference
-  (SCHEME_OBJECT, SCHEME_OBJECT, SCHEME_OBJECT, unsigned long, unsigned int);
-static void add_reference
-  (SCHEME_OBJECT *, SCHEME_OBJECT, SCHEME_OBJECT, unsigned long);
-static void install_cache
-  (SCHEME_OBJECT, SCHEME_OBJECT, unsigned long, unsigned int);
-static void install_operator_cache
-  (SCHEME_OBJECT, SCHEME_OBJECT, unsigned long);
 static unsigned long update_cache_refs_space
-  (SCHEME_OBJECT, SCHEME_OBJECT);
+  (SCHEME_OBJECT, SCHEME_OBJECT, SCHEME_OBJECT);
+static unsigned long update_cache_refs_space_1
+  (SCHEME_OBJECT, unsigned int, SCHEME_OBJECT, SCHEME_OBJECT);
 static long update_cache_references
-  (SCHEME_OBJECT, SCHEME_OBJECT *, SCHEME_OBJECT);
-static unsigned long ref_pairs_to_move
-  (SCHEME_OBJECT *, SCHEME_OBJECT, unsigned long *);
-static void move_ref_pairs
-  (SCHEME_OBJECT, SCHEME_OBJECT, unsigned int, SCHEME_OBJECT);
-static int move_ref_pair_p
-  (SCHEME_OBJECT, SCHEME_OBJECT);
+  (SCHEME_OBJECT, SCHEME_OBJECT *, SCHEME_OBJECT, SCHEME_OBJECT);
 static SCHEME_OBJECT * find_binding_cell
   (SCHEME_OBJECT, SCHEME_OBJECT, SCHEME_OBJECT *);
 static SCHEME_OBJECT * scan_frame
@@ -162,8 +159,6 @@ static SCHEME_OBJECT * scan_procedure_bindings
   (SCHEME_OBJECT, SCHEME_OBJECT, SCHEME_OBJECT, int);
 static unsigned long count_references
   (SCHEME_OBJECT *);
-static SCHEME_OBJECT * find_references_named
-  (SCHEME_OBJECT *, SCHEME_OBJECT);
 static void update_assignment_references
   (SCHEME_OBJECT);
 static long guarantee_cache
@@ -172,8 +167,37 @@ static void update_clone
   (SCHEME_OBJECT);
 static long make_cache
   (SCHEME_OBJECT, SCHEME_OBJECT, SCHEME_OBJECT, SCHEME_OBJECT *);
+
+#ifdef CC_SUPPORT_P
+
+static long update_uuo_links
+  (SCHEME_OBJECT, SCHEME_OBJECT);
+static void move_all_references
+  (SCHEME_OBJECT, SCHEME_OBJECT, unsigned int);
+static long add_cache_reference
+  (SCHEME_OBJECT, SCHEME_OBJECT, SCHEME_OBJECT, unsigned long, unsigned int);
+static void add_reference
+  (SCHEME_OBJECT *, SCHEME_OBJECT, SCHEME_OBJECT, unsigned long);
+static void install_cache
+  (SCHEME_OBJECT, SCHEME_OBJECT, unsigned long, unsigned int);
+static void install_operator_cache
+  (SCHEME_OBJECT, SCHEME_OBJECT, unsigned long);
+static unsigned long ref_pairs_to_move
+  (SCHEME_OBJECT *, SCHEME_OBJECT, SCHEME_OBJECT);
+static void delete_ref_pairs
+  (SCHEME_OBJECT, unsigned int, SCHEME_OBJECT, SCHEME_OBJECT);
+static void move_ref_pairs
+  (SCHEME_OBJECT, SCHEME_OBJECT, unsigned int, SCHEME_OBJECT, SCHEME_OBJECT);
+static SCHEME_OBJECT * new_alist_entry
+  (SCHEME_OBJECT *, SCHEME_OBJECT);
+static int move_ref_pair_p
+  (SCHEME_OBJECT, SCHEME_OBJECT);
+static SCHEME_OBJECT * find_references_named
+  (SCHEME_OBJECT *, SCHEME_OBJECT);
 static long make_cache_reference
   (SCHEME_OBJECT, unsigned long, SCHEME_OBJECT *);
+
+#endif
 
 /***** Basic environment manipulation (lookup, assign, define).  *****/
 
@@ -184,15 +208,10 @@ lookup_variable (SCHEME_OBJECT environment, SCHEME_OBJECT symbol,
   SCHEME_OBJECT * cell;
   SCHEME_OBJECT value;
 
-  if (!ENVIRONMENT_P (environment))
+  if (! ((ENVIRONMENT_P (environment)) && (SYMBOL_P (symbol))))
     return (ERR_BAD_FRAME);
 
-  cell
-    = (find_binding_cell (environment,
-			  (((OBJECT_TYPE (symbol)) == TC_VARIABLE)
-			   ? (GET_VARIABLE_SYMBOL (symbol))
-			   : symbol),
-			  0));
+  cell = (find_binding_cell (environment, symbol, 0));
   if (cell == 0)
     return (ERR_UNBOUND_VARIABLE);
 
@@ -330,15 +349,10 @@ long
 assign_variable (SCHEME_OBJECT environment, SCHEME_OBJECT symbol,
 		 SCHEME_OBJECT value, SCHEME_OBJECT * value_ret)
 {
-  if (!ENVIRONMENT_P (environment))
+  if (! ((ENVIRONMENT_P (environment)) || (SYMBOL_P (symbol))))
     return (ERR_BAD_FRAME);
   {
-    SCHEME_OBJECT * cell
-      = (find_binding_cell (environment,
-			    (((OBJECT_TYPE (symbol)) == TC_VARIABLE)
-			     ? (GET_VARIABLE_SYMBOL (symbol))
-			     : symbol),
-			    0));
+    SCHEME_OBJECT * cell = (find_binding_cell (environment, symbol, 0));
     if (cell == 0)
       return (ERR_UNBOUND_VARIABLE);
     return (assign_variable_end (cell, value, value_ret, 0));
@@ -406,12 +420,15 @@ assign_variable_cache (SCHEME_OBJECT cache, SCHEME_OBJECT value,
   (*value_ret) = (MAP_FROM_UNASSIGNED (old_value));
   /* Perform the assignment.  If there are any operator references to
      this variable, update their links.  */
+#ifdef CC_SUPPORT_P
   if (PAIR_P (* (GET_CACHE_OPERATOR_REFERENCES (cache))))
     return (update_uuo_links (cache, (MAP_TO_UNASSIGNED (value))));
+#endif
   SET_CACHE_VALUE (cache, (MAP_TO_UNASSIGNED (value)));
   return (PRIM_DONE);
 }
 
+#ifdef CC_SUPPORT_P
 static long
 update_uuo_links (SCHEME_OBJECT cache, SCHEME_OBJECT new_value)
 {
@@ -431,12 +448,13 @@ update_uuo_links (SCHEME_OBJECT cache, SCHEME_OBJECT new_value)
      });
   return (PRIM_DONE);
 }
+#endif
 
 long
 define_variable (SCHEME_OBJECT environment, SCHEME_OBJECT symbol,
 		 SCHEME_OBJECT value)
 {
-  if (!ENVIRONMENT_P (environment))
+  if (! ((ENVIRONMENT_P (environment)) || (SYMBOL_P (symbol))))
     return (ERR_BAD_FRAME);
 
   /* If there is already a binding, just assign to it.  */
@@ -444,7 +462,11 @@ define_variable (SCHEME_OBJECT environment, SCHEME_OBJECT symbol,
     SCHEME_OBJECT * cell = (scan_frame (environment, symbol, 1));
     SCHEME_OBJECT old_value;
     if (cell != 0)
-      return (assign_variable_end (cell, value, (&old_value), 1));
+      {
+	if (GLOBAL_FRAME_P (environment))
+	  strengthen_symbol (symbol);
+	return (assign_variable_end (cell, value, (&old_value), 1));
+      }
   }
 
   /* At this point, we know that environment can't be the global
@@ -472,7 +494,7 @@ define_variable (SCHEME_OBJECT environment, SCHEME_OBJECT symbol,
     GC_CHECK
       (2
        + ((old_cache != SHARP_F)
-	  ? (update_cache_refs_space (old_cache, environment))
+	  ? (update_cache_refs_space (old_cache, environment, symbol))
 	  : 0));
 
     /* Create the binding.  */
@@ -484,7 +506,7 @@ define_variable (SCHEME_OBJECT environment, SCHEME_OBJECT symbol,
     return
       ((old_cache != SHARP_F)
        ? (update_cache_references
-	  (old_cache, (PAIR_CDR_LOC (pair)), environment))
+	  (old_cache, (PAIR_CDR_LOC (pair)), environment, symbol))
        : PRIM_DONE);
   }
 }
@@ -548,9 +570,11 @@ link_variables (SCHEME_OBJECT target_environment, SCHEME_OBJECT target_symbol,
   SCHEME_OBJECT * source_cell;
   trap_kind_t source_kind;
   SCHEME_OBJECT * target_cell;
-  
+
   if (! ((ENVIRONMENT_P (target_environment))
-	 && (ENVIRONMENT_P (source_environment))))
+	 && (ENVIRONMENT_P (source_environment))
+	 && (SYMBOL_P (target_symbol))
+	 && (SYMBOL_P (source_symbol))))
     return (ERR_BAD_FRAME);
 
   source_cell = (find_binding_cell (source_environment, source_symbol, 0));
@@ -562,6 +586,12 @@ link_variables (SCHEME_OBJECT target_environment, SCHEME_OBJECT target_symbol,
     return (ERR_UNBOUND_VARIABLE);
 
   target_cell = (scan_frame (target_environment, target_symbol, 1));
+  if (target_cell == source_cell)
+    return (PRIM_DONE);
+
+  if ((target_cell != 0) && (GLOBAL_FRAME_P (target_environment)))
+    strengthen_symbol (target_symbol);
+
   if ((target_cell != 0)
       && ((get_trap_kind (*target_cell)) == TRAP_COMPILER_CACHED))
     {
@@ -577,14 +607,19 @@ link_variables (SCHEME_OBJECT target_environment, SCHEME_OBJECT target_symbol,
 	      * SPACE_PER_UUO_LINK)
 	     + (2 * SPACE_PER_CACHE));
 	  SET_CACHE_VALUE (target_cache, (GET_CACHE_VALUE (source_cache)));
+#ifdef CC_SUPPORT_P
 	  move_all_references
 	    (source_cache, target_cache, CACHE_REFERENCES_LOOKUP);
 	  move_all_references
 	    (source_cache, target_cache, CACHE_REFERENCES_ASSIGNMENT);
 	  move_all_references
 	    (source_cache, target_cache, CACHE_REFERENCES_OPERATOR);
+#endif
 	  update_clone (source_cache);
 	  update_clone (target_cache);
+
+	  /* Make sure both traps share the same cache: */
+	  SET_TRAP_CACHE ((*source_cell), target_cache);
 	}
       else
 	SET_CACHE_VALUE (target_cache, (*source_cell));
@@ -595,19 +630,17 @@ link_variables (SCHEME_OBJECT target_environment, SCHEME_OBJECT target_symbol,
   RETURN_IF_ERROR (guarantee_cache (source_cell));
   return (define_variable (target_environment, target_symbol, (*source_cell)));
 }
-
+
+#ifdef CC_SUPPORT_P
 static void
 move_all_references (SCHEME_OBJECT from_cache, SCHEME_OBJECT to_cache,
 		     unsigned int reference_kind)
 {
-  SCHEME_OBJECT * palist = (GET_CACHE_REFERENCES (to_cache, reference_kind));
-  {
-    SCHEME_OBJECT * pf = (GET_CACHE_REFERENCES (from_cache, reference_kind));
-    (*palist) = (*pf);
-    (*pf) = EMPTY_LIST;
-  }
+  SCHEME_OBJECT * pfrom = (GET_CACHE_REFERENCES (from_cache, reference_kind));
+  SCHEME_OBJECT * pto = (GET_CACHE_REFERENCES (to_cache, reference_kind));
+
   WALK_REFERENCES
-    (palist,
+    (pfrom,
      reference,
      {
        install_cache (to_cache,
@@ -615,7 +648,13 @@ move_all_references (SCHEME_OBJECT from_cache, SCHEME_OBJECT to_cache,
 		      (GET_CACHE_REFERENCE_OFFSET (reference)),
 		      reference_kind);
      });
+
+  while (PAIR_P (*pto))
+    pto = (PAIR_CDR_LOC (*pto));
+  (*pto) = (*pfrom);
+  (*pfrom) = EMPTY_LIST;
 }
+#endif
 
 long
 unbind_variable (SCHEME_OBJECT environment, SCHEME_OBJECT symbol,
@@ -623,6 +662,8 @@ unbind_variable (SCHEME_OBJECT environment, SCHEME_OBJECT symbol,
 {
   SCHEME_OBJECT frame;
   SCHEME_OBJECT * cell = (find_binding_cell (environment, symbol, (&frame)));
+  if (GLOBAL_FRAME_P (frame))
+    weaken_symbol (symbol);
   switch ((cell == 0) ? TRAP_UNBOUND : (get_trap_kind (*cell)))
     {
     case TRAP_UNBOUND:
@@ -644,7 +685,7 @@ unbind_variable (SCHEME_OBJECT environment, SCHEME_OBJECT symbol,
 	  case TRAP_UNBOUND:
 	    (*value_ret) = SHARP_F;
 	    return (PRIM_DONE);
-	    
+
 	  case NON_TRAP_KIND:
 	  case TRAP_UNASSIGNED:
 	  case TRAP_MACRO:
@@ -677,14 +718,9 @@ unbind_cached_variable (SCHEME_OBJECT * cell, SCHEME_OBJECT frame,
   SCHEME_OBJECT cache = (GET_TRAP_CACHE (*cell));
   SCHEME_OBJECT * shadowed_cell
     = (find_binding_cell ((GET_FRAME_PARENT (frame)), symbol, 0));
-  SCHEME_OBJECT dummy_cell = UNBOUND_OBJECT;
-  GC_CHECK (update_cache_refs_space (cache, frame));
+  GC_CHECK (update_cache_refs_space (cache, frame, symbol));
   unbind_variable_1 (cell, frame, symbol);
-  return
-    (update_cache_references
-     (cache,
-      ((shadowed_cell == 0) ? (&dummy_cell) : shadowed_cell),
-      frame));
+  return (update_cache_references (cache, shadowed_cell, frame, symbol));
 }
 
 static void
@@ -714,12 +750,14 @@ unbind_variable_1 (SCHEME_OBJECT * cell,
 
 /***** Interface to compiled code.  *****/
 
+#ifdef CC_SUPPORT_P
+
 long
 compiler_cache_lookup (SCHEME_OBJECT name, SCHEME_OBJECT block,
 		       unsigned long offset)
 {
   return
-    (add_cache_reference ((compiled_block_environment (block)),
+    (add_cache_reference ((cc_block_environment (block)),
 			  name, block, offset,
 			  CACHE_REFERENCES_LOOKUP));
 }
@@ -729,7 +767,7 @@ compiler_cache_assignment (SCHEME_OBJECT name, SCHEME_OBJECT block,
 			   unsigned long offset)
 {
   return
-    (add_cache_reference ((compiled_block_environment (block)),
+    (add_cache_reference ((cc_block_environment (block)),
 			  name, block, offset,
 			  CACHE_REFERENCES_ASSIGNMENT));
 }
@@ -739,7 +777,7 @@ compiler_cache_operator (SCHEME_OBJECT name, SCHEME_OBJECT block,
 			 unsigned long offset)
 {
   return
-    (add_cache_reference ((compiled_block_environment (block)),
+    (add_cache_reference ((cc_block_environment (block)),
 			  name, block, offset,
 			  CACHE_REFERENCES_OPERATOR));
 }
@@ -762,6 +800,8 @@ compiler_var_error (SCHEME_OBJECT cache, SCHEME_OBJECT block,
     ((GET_CACHE_REFERENCES (cache, reference_kind)),
      reference,
      {
+       /* If this reference is in the right block, return the symbol
+	  being referenced.  */
        if ((GET_CACHE_REFERENCE_BLOCK (reference)) == block)
 	 return (PAIR_CAR (PAIR_CAR (*WR_palist)));
      });
@@ -873,7 +913,8 @@ add_cache_reference (SCHEME_OBJECT environment, SCHEME_OBJECT symbol,
 		     SCHEME_OBJECT block, unsigned long offset,
 		     unsigned int reference_kind)
 {
-  SCHEME_OBJECT * cell = (find_binding_cell (environment, symbol, 0));
+  SCHEME_OBJECT frame = 0;
+  SCHEME_OBJECT * cell = (find_binding_cell (environment, symbol, (&frame)));
   SCHEME_OBJECT dummy_cell = UNBOUND_OBJECT;
   if (cell == 0)
     /* There's no binding for the variable, and we don't have access
@@ -881,6 +922,8 @@ add_cache_reference (SCHEME_OBJECT environment, SCHEME_OBJECT symbol,
        we'll install one, but it won't be attached to any environment
        structure.  */
     cell = (&dummy_cell);
+  else if (GLOBAL_FRAME_P (frame))
+    strengthen_symbol (symbol);
   /* This procedure must complete to keep the data structures
      consistent, so we do a GC check in advance to guarantee that all
      of the allocations will finish.  */
@@ -904,35 +947,33 @@ static void
 add_reference (SCHEME_OBJECT * palist,
 	       SCHEME_OBJECT symbol, SCHEME_OBJECT block, unsigned long offset)
 {
-  while (PAIR_P (*palist))
+  SCHEME_OBJECT * prefs = (find_references_named (palist, symbol));
+  if (prefs != 0)
     {
-      if ((PAIR_CAR (PAIR_CAR (*palist))) == symbol)
+      while (PREFS_COND (prefs))
 	{
-	  SCHEME_OBJECT * prefs = (PAIR_CDR_LOC (PAIR_CAR (*palist)));
-	  while (PAIR_P (*prefs))
+	  if ((GET_CACHE_REFERENCE_BLOCK (PAIR_CAR (*prefs))) == SHARP_F)
 	    {
-	      if ((GET_CACHE_REFERENCE_BLOCK (PAIR_CAR (*prefs))) == SHARP_F)
-		{
-		  SET_CACHE_REFERENCE_BLOCK ((PAIR_CAR (*prefs)), block);
-		  SET_CACHE_REFERENCE_OFFSET ((PAIR_CAR (*prefs)), offset);
-		  return;
-		}
-	      prefs = (PAIR_CDR_LOC (*prefs));
+	      /* Reuse this pair.  */
+	      SET_CACHE_REFERENCE_BLOCK ((PAIR_CAR (*prefs)), block);
+	      SET_CACHE_REFERENCE_OFFSET ((PAIR_CAR (*prefs)), offset);
+	      return;
 	    }
-	  {
-	    SCHEME_OBJECT reference;
-	    DIE_IF_ERROR (make_cache_reference (block, offset, (&reference)));
-	    (*prefs) = (cons (reference, EMPTY_LIST));
-	  }
-	  return;
+	  PREFS_FOOTER (prefs);
 	}
-      palist = (PAIR_CDR_LOC (*palist));
+      {
+	SCHEME_OBJECT reference;
+	DIE_IF_ERROR (make_cache_reference (block, offset, (&reference)));
+	(*prefs) = (cons (reference, EMPTY_LIST));
+      }
+      return;
     }
   {
     SCHEME_OBJECT reference;
+    SCHEME_OBJECT alist;
     DIE_IF_ERROR (make_cache_reference (block, offset, (&reference)));
-    (*palist)
-      = (cons ((cons (symbol, (cons (reference, EMPTY_LIST)))), EMPTY_LIST));
+    alist = (*palist);
+    (*palist) = (cons ((cons (symbol, (cons (reference, EMPTY_LIST)))), alist));
   }
 }
 
@@ -943,11 +984,11 @@ install_cache (SCHEME_OBJECT cache, SCHEME_OBJECT block, unsigned long offset,
   switch (reference_kind)
     {
     case CACHE_REFERENCES_LOOKUP:
-      store_variable_cache (cache, block, offset);
+      write_variable_cache (cache, block, offset);
       break;
 
     case CACHE_REFERENCES_ASSIGNMENT:
-      store_variable_cache
+      write_variable_cache
 	((((GET_CACHE_CLONE (cache)) != SHARP_F)
 	  ? (GET_CACHE_CLONE (cache))
 	  : cache),
@@ -970,132 +1011,172 @@ install_operator_cache (SCHEME_OBJECT cache,
 			SCHEME_OBJECT block, unsigned long offset)
 {
   SCHEME_OBJECT value = (GET_CACHE_VALUE (cache));
-  DIE_IF_ERROR
-    ((REFERENCE_TRAP_P (value))
-     ? (make_fake_uuo_link (cache, block, offset))
-     : (make_uuo_link (value, cache, block, offset)));
+  DIE_IF_ERROR (make_uuo_link (value, cache, block, offset));
 }
+
+#endif /* CC_SUPPORT_P */
 
 static unsigned long
-update_cache_refs_space (SCHEME_OBJECT from_cache, SCHEME_OBJECT environment)
+update_cache_refs_space (SCHEME_OBJECT from_cache, SCHEME_OBJECT environment,
+			 SCHEME_OBJECT symbol)
 {
-  unsigned long n_names = 0;
-  unsigned long n_lookups
-    = (ref_pairs_to_move ((GET_CACHE_LOOKUP_REFERENCES (from_cache)),
-			  environment, (&n_names)));
-  unsigned long n_assignments
-    = (ref_pairs_to_move ((GET_CACHE_ASSIGNMENT_REFERENCES (from_cache)),
-			  environment, (&n_names)));
-  unsigned long n_operators
-    = (ref_pairs_to_move ((GET_CACHE_OPERATOR_REFERENCES (from_cache)),
-			  environment, (&n_names)));
-
-  /* No references need to be updated.  */
-  if ((n_lookups == 0) && (n_assignments == 0) && (n_operators == 0))
-    return (PRIM_DONE);
-
+#ifdef CC_SUPPORT_P
   return
-    ((n_operators * SPACE_PER_UUO_LINK)
-     + (n_names * 4)
-     + (3 * SPACE_PER_CACHE));
+    ((update_cache_refs_space_1
+      (from_cache, CACHE_REFERENCES_LOOKUP, environment, symbol))
+     + (update_cache_refs_space_1
+	(from_cache, CACHE_REFERENCES_ASSIGNMENT, environment, symbol))
+     + (update_cache_refs_space_1
+	(from_cache, CACHE_REFERENCES_OPERATOR, environment, symbol)));
+#else
+  return (0);
+#endif
 }
 
+/* Generate a conservative estimate of the space needed to move some
+   cache refs from one cache to another.  */
+
+static unsigned long
+update_cache_refs_space_1 (SCHEME_OBJECT from_cache, unsigned int kind,
+			   SCHEME_OBJECT environment, SCHEME_OBJECT symbol)
+{
+  SCHEME_OBJECT * from_palist = (GET_CACHE_REFERENCES (from_cache, kind));
+  unsigned long n_refs = (ref_pairs_to_move (from_palist, environment, symbol));
+  unsigned long result = 0;
+  if (n_refs > 0)
+    {
+      /* Space for new cache and new alist entry, if needed.  */
+      result += (SPACE_PER_CACHE + 4);
+      if (kind == CACHE_REFERENCES_OPERATOR)
+	/* space for new trampolines, if needed.  */
+	result += (n_refs * SPACE_PER_UUO_LINK);
+    }
+  return (result);
+}
+
 static long
 update_cache_references (SCHEME_OBJECT from_cache, SCHEME_OBJECT * to_cell,
-			 SCHEME_OBJECT environment)
+			 SCHEME_OBJECT environment, SCHEME_OBJECT symbol)
 {
-  DIE_IF_ERROR (guarantee_cache (to_cell));
-  {
-    SCHEME_OBJECT to_cache = (GET_TRAP_CACHE (*to_cell));
-    move_ref_pairs
-      (from_cache, to_cache, CACHE_REFERENCES_LOOKUP, environment);
-    move_ref_pairs
-      (from_cache, to_cache, CACHE_REFERENCES_ASSIGNMENT, environment);
-    move_ref_pairs
-      (from_cache, to_cache, CACHE_REFERENCES_OPERATOR, environment);
-    update_clone (to_cache);
-  }
+  if (to_cell != 0)
+    {
+      DIE_IF_ERROR (guarantee_cache (to_cell));
+      {
+	SCHEME_OBJECT to_cache = (GET_TRAP_CACHE (*to_cell));
+#ifdef CC_SUPPORT_P
+	move_ref_pairs
+	  (from_cache, to_cache, CACHE_REFERENCES_LOOKUP,
+	   environment, symbol);
+	move_ref_pairs
+	  (from_cache, to_cache, CACHE_REFERENCES_ASSIGNMENT,
+	   environment, symbol);
+	move_ref_pairs
+	  (from_cache, to_cache, CACHE_REFERENCES_OPERATOR,
+	   environment, symbol);
+#endif
+	update_clone (to_cache);
+      }
+    }
+#ifdef CC_SUPPORT_P
+  else
+    {
+      delete_ref_pairs
+	(from_cache, CACHE_REFERENCES_LOOKUP, environment, symbol);
+      delete_ref_pairs
+	(from_cache, CACHE_REFERENCES_ASSIGNMENT, environment, symbol);
+      delete_ref_pairs
+	(from_cache, CACHE_REFERENCES_OPERATOR, environment, symbol);
+    }
+#endif
   update_clone (from_cache);
   return (PRIM_DONE);
 }
 
+#ifdef CC_SUPPORT_P
+
 static unsigned long
 ref_pairs_to_move (SCHEME_OBJECT * palist, SCHEME_OBJECT environment,
-		   unsigned long * n_names_ret)
+		   SCHEME_OBJECT symbol)
 {
+  SCHEME_OBJECT * prefs = (find_references_named (palist, symbol));
   unsigned long n_refs = 0;
-  while (PAIR_P (*palist))
-    {
-      int any_moved_p = 0;
-      SCHEME_OBJECT * prefs = (PAIR_CDR_LOC (PAIR_CAR (*palist)));
-      while (PAIR_P (*prefs))
-	if ((GET_CACHE_REFERENCE_BLOCK (PAIR_CAR (*prefs))) == SHARP_F)
-	  (*prefs) = (PAIR_CDR (*prefs));
-	else
-	  {
-	    if (move_ref_pair_p ((*prefs), environment))
-	      {
-		n_refs += 1;
-		any_moved_p = 1;
-	      }
-	    prefs = (PAIR_CDR_LOC (*prefs));
-	  }
-      if (any_moved_p)
-	(*n_names_ret) += 1;
-      palist = (PAIR_CDR_LOC (*palist));
-    }
+  if (prefs != 0)
+    while (PREFS_COND (prefs))
+      {
+	PREFS_HEADER (prefs);
+	if (move_ref_pair_p ((*prefs), environment))
+	  n_refs += 1;
+	PREFS_FOOTER (prefs);
+      }
   return (n_refs);
 }
 
+static void
+delete_ref_pairs (SCHEME_OBJECT from_cache, unsigned int kind,
+		  SCHEME_OBJECT environment, SCHEME_OBJECT symbol)
+{
+  SCHEME_OBJECT * from_palist = (GET_CACHE_REFERENCES (from_cache, kind));
+  SCHEME_OBJECT * from_prefs = (find_references_named (from_palist, symbol));
+  if (from_prefs != 0)
+    while (PREFS_COND (from_prefs))
+      {
+	PREFS_HEADER (from_prefs);
+	if (move_ref_pair_p ((*from_prefs), environment))
+	  {
+	    (*from_prefs) = (PAIR_CDR (*from_prefs));
+	    continue;
+	  }
+	PREFS_FOOTER (from_prefs);
+      }
+}
+
 static void
 move_ref_pairs (SCHEME_OBJECT from_cache, SCHEME_OBJECT to_cache,
-		unsigned int reference_kind, SCHEME_OBJECT environment)
+		unsigned int reference_kind, SCHEME_OBJECT environment,
+		SCHEME_OBJECT symbol)
 {
   SCHEME_OBJECT * from_palist
     = (GET_CACHE_REFERENCES (from_cache, reference_kind));
   SCHEME_OBJECT * to_palist
     = (GET_CACHE_REFERENCES (to_cache, reference_kind));
-  while (PAIR_P (*from_palist))
-    {
-      SCHEME_OBJECT * from_prefs = (PAIR_CDR_LOC (PAIR_CAR (*from_palist)));
-      SCHEME_OBJECT symbol = (PAIR_CAR (PAIR_CAR (*from_palist)));
-      SCHEME_OBJECT * to_prefs = (find_references_named (to_palist, symbol));
-      while (PAIR_P (*from_prefs))
+  SCHEME_OBJECT * from_prefs = (find_references_named (from_palist, symbol));
+  SCHEME_OBJECT * to_prefs = (find_references_named (to_palist, symbol));
+  if (from_prefs != 0)
+    while (PREFS_COND (from_prefs))
+      {
+	PREFS_HEADER (from_prefs);
 	if (move_ref_pair_p ((*from_prefs), environment))
 	  {
 	    SCHEME_OBJECT p = (*from_prefs);
 	    (*from_prefs) = (PAIR_CDR (p));
 	    if (to_prefs == 0)
-	      {
-		SCHEME_OBJECT p2;
-		SET_PAIR_CDR (p, EMPTY_LIST);
-		p2 = (cons ((cons (symbol, p)), (*to_palist)));
-		(*to_palist) = p2;
-	      }
-	    else
-	      {
-		SET_PAIR_CDR (p, (*to_prefs));
-		(*to_prefs) = p;
-	      }
+	      to_prefs = (new_alist_entry (to_palist, symbol));
+	    SET_PAIR_CDR (p, (*to_prefs));
+	    (*to_prefs) = p;
 	    install_cache (to_cache,
 			   (GET_CACHE_REFERENCE_BLOCK (PAIR_CAR (p))),
 			   (GET_CACHE_REFERENCE_OFFSET (PAIR_CAR (p))),
 			   reference_kind);
+	    continue;
 	  }
-	else
-	  from_prefs = (PAIR_CDR_LOC (*from_prefs));
-      if (PAIR_P (PAIR_CDR (PAIR_CAR (*from_palist))))
-	from_palist = (PAIR_CDR_LOC (*from_palist));
-      else
-	(*from_palist) = (PAIR_CDR (*from_palist));
-    }
+	PREFS_FOOTER (from_prefs);
+      }
+}
+
+static SCHEME_OBJECT *
+new_alist_entry (SCHEME_OBJECT * to_palist, SCHEME_OBJECT symbol)
+{
+  SCHEME_OBJECT entry = (cons (symbol, EMPTY_LIST));
+  SCHEME_OBJECT head = (*to_palist);
+  (*to_palist) = (cons (entry, head));
+  return (PAIR_CDR_LOC (entry));
 }
 
 static int
 move_ref_pair_p (SCHEME_OBJECT ref_pair, SCHEME_OBJECT ancestor)
 {
   SCHEME_OBJECT descendant
-    = (compiled_block_environment
+    = (cc_block_environment
        (GET_CACHE_REFERENCE_BLOCK (PAIR_CAR (ref_pair))));
   while (PROCEDURE_FRAME_P (descendant))
     {
@@ -1105,6 +1186,8 @@ move_ref_pair_p (SCHEME_OBJECT ref_pair, SCHEME_OBJECT ancestor)
     }
   return (descendant == ancestor);
 }
+
+#endif /* CC_SUPPORT_P */
 
 /***** Utilities *****/
 
@@ -1200,6 +1283,7 @@ count_references (SCHEME_OBJECT * palist)
   return (n_references);
 }
 
+#ifdef CC_SUPPORT_P
 static SCHEME_OBJECT *
 find_references_named (SCHEME_OBJECT * palist, SCHEME_OBJECT symbol)
 {
@@ -1211,10 +1295,12 @@ find_references_named (SCHEME_OBJECT * palist, SCHEME_OBJECT symbol)
     }
   return (0);
 }
+#endif
 
 static void
 update_assignment_references (SCHEME_OBJECT cache)
 {
+#ifdef CC_SUPPORT_P
   SCHEME_OBJECT reference_cache
     = (((GET_CACHE_CLONE (cache)) != SHARP_F)
        ? (GET_CACHE_CLONE (cache))
@@ -1223,11 +1309,12 @@ update_assignment_references (SCHEME_OBJECT cache)
     ((GET_CACHE_ASSIGNMENT_REFERENCES (cache)),
      reference,
      {
-       store_variable_cache
+       write_variable_cache
 	 (reference_cache,
 	  (GET_CACHE_REFERENCE_BLOCK (reference)),
 	  (GET_CACHE_REFERENCE_OFFSET (reference)));
      });
+#endif
 }
 
 static long
@@ -1294,6 +1381,7 @@ make_cache (SCHEME_OBJECT value, SCHEME_OBJECT clone, SCHEME_OBJECT references,
   return (PRIM_DONE);
 }
 
+#ifdef CC_SUPPORT_P
 static long
 make_cache_reference (SCHEME_OBJECT block, unsigned long offset,
 		      SCHEME_OBJECT * ref_ret)
@@ -1304,3 +1392,4 @@ make_cache_reference (SCHEME_OBJECT block, unsigned long offset,
   (*ref_ret) = (MAKE_POINTER_OBJECT (TC_WEAK_CONS, (Free - 2)));
   return (PRIM_DONE);
 }
+#endif
