@@ -2,8 +2,8 @@
 
 # Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
 #     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-#     2005, 2006, 2007, 2008, 2009, 2010 Massachusetts Institute of
-#     Technology
+#     2005, 2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute
+#     of Technology
 #
 # This file is part of MIT/GNU Scheme.
 #
@@ -24,13 +24,37 @@
 
 # Processing to simulate m4 accepting definition arguments.
 
+set -e
+
+if [ $# -le 1 ]; then
+  printf 'Usage: %s m4 <file/definition> ...\n' >&2
+  exit 1
+fi
+
+M4="${1}"
+shift
+
+TMP_FILE="m4.tmp"
+
+clean ()
+{
+  rm -f "${TMP_FILE}"
+}
+
+run_m4 ()
+{
+  ${M4} && clean
+}
+
+trap clean EXIT INT QUIT TERM
+rm -f "${TMP_FILE}"
+touch "${TMP_FILE}"
+
 if [ $# = 0 ]
 then
-  sed -e '/^#/D' | m4 | sed -e 's/@/$/g' -e 's/^$//'
+  sed -e '/^#/D' | run_m4 | sed -e 's/@/$/g' -e 's/^$//'
 else
-  TMP_FILE="m4.tmp"
   SEEN_INPUT=0
-  rm -f "${TMP_FILE}"
   while [ $# != 0 ]; do
     if [ "${1}" = "-P" ]; then
       echo "define(${2})" >> "${TMP_FILE}"
@@ -44,6 +68,13 @@ else
   if [ ${SEEN_INPUT} -eq 0 ]; then
     sed -e '/^#/D' >> "${TMP_FILE}"
   fi
-  m4 < "${TMP_FILE}" | sed -e 's/@/$/g' -e 's/^$//'
-  rm -f "${TMP_FILE}"
+  run_m4 < "${TMP_FILE}" | sed -e 's/@/$/g' -e 's/^$//'
+fi
+
+# If m4 was successful, run_m4 has deleted the temporary file.  If
+# not, report the failure; exiting will have the effect of running
+# `clean', which will delete the temporary file.
+
+if [ -f "${TMP_FILE}" ]; then
+  exit 1
 fi
