@@ -1,8 +1,9 @@
 /* -*-C-*-
 
-$Id: nttrap.h,v 1.7 2003/02/14 18:28:21 cph Exp $
-
-Copyright (c) 1992-1999 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,14 +19,14 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 */
 
 #ifndef SCM_NTTRAP_H
 #define SCM_NTTRAP_H
-
+
 enum trap_state
 {
   trap_state_trapped,
@@ -37,6 +38,6 @@ enum trap_state
   trap_state_exitting_hard
 };
 
-extern enum trap_state EXFUN (OS_set_trap_state, (enum trap_state state));
+extern enum trap_state OS_set_trap_state (enum trap_state state);
 
 #endif /* SCM_NTTRAP_H */
