@@ -1,10 +1,9 @@
 /* -*-C-*-
 
-$Id: os2term.c,v 1.8 2007/01/05 21:19:25 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -75,12 +74,12 @@ OS_terminal_state_size (void)
 }
 
 void
-OS_terminal_get_state (Tchannel channel, PTR statep)
+OS_terminal_get_state (Tchannel channel, void * statep)
 {
 }
 
 void
-OS_terminal_set_state (Tchannel channel, PTR statep)
+OS_terminal_set_state (Tchannel channel, void * statep)
 {
 }
 
