@@ -1,10 +1,9 @@
 /* -*-C-*-
 
-$Id: ux.h,v 1.88 2008/01/30 20:02:21 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -74,6 +73,10 @@ USA.
 
 #if defined(__netbsd__) || defined(__NetBSD__)
 #  define SYSTEM_VARIANT "NetBSD"
+#  include <sys/param.h>
+#  if defined(__NetBSD_Version__) && __NetBSD_Version__ >= 200000000
+#    define HAVE_SIGACTION_SIGINFO_SIGNALS
+#  endif
 #endif
 
 #ifdef _NEXTOS
@@ -211,6 +214,35 @@ USA.
 #  endif
 #endif
 
+#ifdef HAVE_SYS_TIMEX_H
+#  include <sys/timex.h>
+#endif
+
+/* This detects both the NTP system calls found in BSD systems (NetBSD,
+   FreeBSD) and the NTP system calls found in Linux systems.  What is
+   found on other systems, I don't know.  We use this to get at the
+   system's record of the UTC - TAI offset.  */
+
+#ifdef HAVE_NTP_GETTIME
+#ifdef HAVE_STRUCT_NTPTIMEVAL
+#ifdef HAVE_NTPTIMEVAL_TAI
+#ifdef HAVE_NTPTIMEVAL_TIME_TV_NSEC
+#  define HAVE_BSD_NTP
+#endif
+#endif
+#endif
+#endif
+
+#ifdef HAVE_NTP_ADJTIME
+#ifdef HAVE_STRUCT_TIMEX
+#ifdef HAVE_TIMEX_TAI
+#ifdef HAVE_TIMEX_TIME_TV_USEC
+#  define HAVE_LINUX_NTP
+#endif
+#endif
+#endif
+#endif
+
 #ifdef HAVE_UTIME_H
 #  include <utime.h>
 #else
@@ -272,6 +304,10 @@ USA.
 typedef RETSIGTYPE Tsignal_handler_result;
 
 #ifdef _POSIX_REALTIME_SIGNALS
+#  define HAVE_SIGACTION_SIGINFO_SIGNALS
+#endif
+
+#ifdef HAVE_SIGACTION_SIGINFO_SIGNALS
    typedef void (*Tsignal_handler) (int, siginfo_t *, void *);
 #else
    typedef RETSIGTYPE (*Tsignal_handler) (int);
@@ -452,14 +488,18 @@ typedef RETSIGTYPE Tsignal_handler_result;
 #define UX_bind bind
 #define UX_chdir chdir
 #define UX_chmod chmod
+#define UX_clock_gettime clock_gettime
 #define UX_close close
 #define UX_connect connect
 #define UX_ctime ctime
 #define UX_dup dup
 #define UX_fcntl fcntl
+#define UX_fdatasync fdatasync
 #define UX_free free
 #define UX_fstat fstat
 #define UX_fstatfs fstatfs
+#define UX_fsync fsync
+#define UX_fsync_range fsync_range
 #define UX_ftruncate ftruncate
 #define UX_getegid getegid
 #define UX_getenv getenv
@@ -484,6 +524,8 @@ typedef RETSIGTYPE Tsignal_handler_result;
 #define UX_malloc malloc
 #define UX_mknod mknod
 #define UX_mktime mktime
+#define UX_ntp_adjtime ntp_adjtime
+#define UX_ntp_gettime ntp_gettime
 #define UX_open open
 #define UX_pause pause
 #define UX_pipe pipe
@@ -499,6 +541,7 @@ typedef RETSIGTYPE Tsignal_handler_result;
 #define UX_stat stat
 #define UX_statfs statfs
 #define UX_symlink symlink
+#define UX_sync_file_range sync_file_range
 #define UX_system system
 #define UX_time time
 #define UX_times times
@@ -579,6 +622,13 @@ typedef RETSIGTYPE Tsignal_handler_result;
    extern unsigned long UX_getpagesize (void);
 #  define EMULATE_GETPAGESIZE
 #endif
+
+#ifdef HAVE_CLOSEFROM
+#  define UX_closefrom closefrom
+#else
+   extern int UX_closefrom (int);
+#  define EMULATE_CLOSEFROM
+#endif
 
 /* poll is somewhat busted on Mac OSX 10.4 (Tiger), so use select.  */
 #ifdef __APPLE__
@@ -707,17 +757,17 @@ extern int UX_terminal_set_state (int, Ttty_state *);
 #    ifdef SETPGRP_VOID
 #      define UX_setsid setpgrp
 #    else
-         extern pid_t UX_setsid (void);
-#        define EMULATE_SETSID
+	 extern pid_t UX_setsid (void);
+#	 define EMULATE_SETSID
 #    endif
 #    ifdef HAVE_SETPGRP2
 #      define UX_setpgid setpgrp2
 #    else
 #      ifdef SETPGRP_VOID
-         extern int UX_setpgid (pid_t, pid_t);
-#        define EMULATE_SETPGID
+	 extern int UX_setpgid (pid_t, pid_t);
+#	 define EMULATE_SETPGID
 #      else
-#        define UX_setpgid setpgrp
+#	define UX_setpgid setpgrp
 #      endif
 #    endif
 #  endif
@@ -848,26 +898,39 @@ extern int UX_sigsuspend (const sigset_t *);
 #endif /* not _POSIX_VERSION */
 
 extern void UX_prim_check_errno (enum syscall_names name);
+extern void UX_prim_check_fd_errno (enum syscall_names name);
+extern bool UX_out_of_files_p;
 
-#define STD_VOID_SYSTEM_CALL(name, expression)				\
-{									\
-  while ((expression) < 0)						\
-    if (errno != EINTR)							\
-      error_system_call (errno, (name));				\
-}
+#define STD_VOID_SYSTEM_CALL(name, expression)	\
+  do {						\
+    while ((expression) < 0)			\
+      UX_prim_check_errno (name);		\
+  } while (0)
 
-#define STD_UINT_SYSTEM_CALL(name, result, expression)			\
-{									\
-  while (((result) = (expression)) < 0)					\
-    if (errno != EINTR)							\
-      error_system_call (errno, (name));				\
-}
+#define STD_UINT_SYSTEM_CALL(name, result, expression)	\
+  do {							\
+    while (((result) = (expression)) < 0)		\
+      UX_prim_check_errno (name);			\
+  } while (0)
 
-#define STD_PTR_SYSTEM_CALL(name, result, expression)			\
-{									\
-  while (((result) = (expression)) == 0)				\
-    if (errno != EINTR)							\
-      error_system_call (errno, (name));				\
-}
+#define STD_PTR_SYSTEM_CALL(name, result, expression)	\
+  do {							\
+    while (((result) = (expression)) == 0)		\
+      UX_prim_check_errno (name);			\
+  } while (0)
+
+#define STD_FD_SYSTEM_CALL(name, result, expression)	\
+  do {							\
+    while (((result) = (expression)) < 0)		\
+      UX_prim_check_fd_errno (name);			\
+    UX_out_of_files_p = false;				\
+  } while (0)
+
+#define STD_FD_VOID_SYSTEM_CALL(name, expression)	\
+  do {							\
+    while ((expression) < 0)				\
+      UX_prim_check_fd_errno (name);			\
+    UX_out_of_files_p = false;				\
+  } while (0)
 
 #endif /* SCM_UX_H */
