@@ -1,10 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: defstr.scm,v 14.59 2008/02/14 00:45:17 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -30,7 +29,7 @@ USA.
 
 (declare (usual-integrations))
 
-#| 
+#|
 
 This macro works like the Common Lisp `defstruct' with the following
 differences:
@@ -58,6 +57,9 @@ differences:
   a procedure of two arguments (the unparser state and the structure
   instance) rather than three as in Common Lisp.
 
+* There is an additional option PRINT-ENTITY-PROCEDURE, used to print
+  an entity whose extra object is a structure instance.
+
 * By default, named structures are tagged with a unique object of some
   kind.  In Common Lisp, the structures are tagged with symbols, but
   that depends on the Common Lisp package system to help generate
@@ -75,37 +77,35 @@ differences:
 
 |#
 
-(define-expander 'DEFINE-STRUCTURE system-global-environment
-  (lambda (form environment closing-environment)
-    (if (not (and (pair? (cdr form)) (list? (cddr form))))
-	(error "Ill-formed special form:" form))
-    (make-syntactic-closure closing-environment '()
-      (let ((name-and-options (cadr form))
-	    (slot-descriptions (cddr form)))
+(define-syntax define-structure
+  (sc-macro-transformer
+   (lambda (form use-environment)
+     (syntax-check '(KEYWORD + DATUM) form)
+     (capture-syntactic-environment
+      (lambda (closing-environment)
 	(let ((structure
-	       (call-with-values
-		   (lambda ()
+	       (receive (name options)
+		   (let ((name-and-options (cadr form)))
 		     (if (pair? name-and-options)
 			 (values (car name-and-options) (cdr name-and-options))
 			 (values name-and-options '())))
-		 (lambda (name options)
-		   (if (not (symbol? name))
-		       (error "Structure name must be a symbol:" name))
-		   (if (not (list? options))
-		       (error "Structure options must be a list:" options))
-		   (let ((context
-			  (make-parser-context name
-					       environment
-					       closing-environment)))
-		     (parse/options options
-				    (parse/slot-descriptions slot-descriptions)
-				    context))))))
+		 (if (not (symbol? name))
+		     (error "Structure name must be a symbol:" name))
+		 (if (not (list? options))
+		     (error "Structure options must be a list:" options))
+		 (let ((context
+			(make-parser-context name
+					     use-environment
+					     closing-environment)))
+		   (parse/options options
+				  (parse/slot-descriptions (cddr form))
+				  context)))))
 	  `(BEGIN ,@(type-definitions structure)
 		  ,@(constructor-definitions structure)
 		  ,@(accessor-definitions structure)
 		  ,@(modifier-definitions structure)
 		  ,@(predicate-definitions structure)
-		  ,@(copier-definitions structure)))))))
+		  ,@(copier-definitions structure))))))))
 
 ;;;; Parse options
 
@@ -118,6 +118,8 @@ differences:
 	  (copier-option (find-option 'COPIER options))
 	  (predicate-option (find-option 'PREDICATE options))
 	  (print-procedure-option (find-option 'PRINT-PROCEDURE options))
+	  (print-entity-procedure-option
+	   (find-option 'PRINT-ENTITY-PROCEDURE options))
 	  (type-option (find-option 'TYPE options))
 	  (type-descriptor-option (find-option 'TYPE-DESCRIPTOR options))
 	  (named-option (find-option 'NAMED options))
@@ -136,7 +138,8 @@ differences:
 	    (check-for-illegal-untyped named-option initial-offset-option))
 	(if (not tagged?)
 	    (check-for-illegal-untagged predicate-option
-					print-procedure-option))
+					print-procedure-option
+					print-entity-procedure-option))
 	(do ((slots slots (cdr slots))
 	     (index (if tagged? (+ offset 1) offset) (+ index 1)))
 	    ((not (pair? slots)))
@@ -163,6 +166,9 @@ differences:
 				(option/argument print-procedure-option)
 				(and type-option
 				     (default-unparser-text context)))
+			    (if print-entity-procedure-option
+				(option/argument print-entity-procedure-option)
+				#f)
 			    (if type-option
 				(option/argument type-option)
 				'RECORD)
@@ -215,7 +221,9 @@ differences:
     (if initial-offset-option
 	(lose initial-offset-option))))
 
-(define (check-for-illegal-untagged predicate-option print-procedure-option)
+(define (check-for-illegal-untagged predicate-option
+				    print-procedure-option
+				    print-entity-procedure-option)
   (let ((test
 	 (lambda (option)
 	   (if (and option
@@ -225,7 +233,8 @@ differences:
 	       (error "Structure option illegal for unnamed structure:"
 		      (option/original option))))))
     (test predicate-option)
-    (test print-procedure-option)))
+    (test print-procedure-option)
+    (test print-entity-procedure-option)))
 
 (define (compute-constructors constructor-options
 			      keyword-constructor-options
@@ -257,7 +266,7 @@ differences:
       (and (identifier? object)
 	   (there-exists? false-expression-names
 	     (lambda (name)
-	       (identifier=? (parser-context/environment context)
+	       (identifier=? (parser-context/use-environment context)
 			     object
 			     (parser-context/closing-environment context)
 			     name))))))
@@ -429,6 +438,13 @@ differences:
       (lambda (arg)
 	`(PRINT-PROCEDURE ,(if (false-expression? arg context) #f arg))))))
 
+(define-option 'PRINT-ENTITY-PROCEDURE #f
+  (lambda (option context)
+    (one-required-argument option
+      (lambda (arg)
+	`(PRINT-ENTITY-PROCEDURE
+	  ,(if (false-expression? arg context) #f arg))))))
+
 (define-option 'TYPE #f
   (lambda (option context)
     context
@@ -542,9 +558,9 @@ differences:
 
 (define-record-type <structure>
     (make-structure context conc-name constructors keyword-constructors copier
-		    predicate print-procedure physical-type named?
-		    type-descriptor tag-expression safe-accessors? offset
-		    slots)
+		    predicate print-procedure print-entity-procedure
+		    physical-type named? type-descriptor tag-expression
+		    safe-accessors? offset slots)
     structure?
   (context structure/context)
   (conc-name structure/conc-name)
@@ -553,6 +569,7 @@ differences:
   (copier structure/copier)
   (predicate structure/predicate)
   (print-procedure structure/print-procedure)
+  (print-entity-procedure structure/print-entity-procedure)
   (physical-type structure/physical-type)
   (named? structure/tagged?)
   (type-descriptor structure/type-descriptor)
@@ -565,10 +582,10 @@ differences:
   (eq? (structure/physical-type structure) 'RECORD))
 
 (define-record-type <parser-context>
-    (make-parser-context name environment closing-environment)
+    (make-parser-context name use-environment closing-environment)
     parser-context?
   (name parser-context/name)
-  (environment parser-context/environment)
+  (use-environment parser-context/use-environment)
   (closing-environment parser-context/closing-environment))
 
 (define-record-type <option>
@@ -597,7 +614,7 @@ differences:
 		(parser-context/closing-environment context)))
 
 (define (close name context)
-  (close-syntax name (parser-context/environment context)))
+  (close-syntax name (parser-context/use-environment context)))
 
 (define (accessor-definitions structure)
   (let ((context (structure/context structure)))
@@ -701,8 +718,8 @@ differences:
 			(optional (map name->slot optional))
 			(rest (and rest (name->slot rest))))
 		    (map (lambda (slot)
-			   (let* ((name (slot/name slot))
-				  (dv (default-value-expr structure name)))
+			   (let ((name (slot/name slot))
+				 (dv (default-value-expr structure slot)))
 			     (cond ((or (memq slot required)
 					(eq? slot rest))
 				    name)
@@ -710,7 +727,7 @@ differences:
 				    `(IF (DEFAULT-OBJECT? ,name) ,dv ,name))
 				   (else dv))))
 			 (structure/slots structure)))))))))))
-
+
 (define (make-constructor structure name lambda-list generate-body)
   (let* ((context (structure/context structure))
 	(tag-expression (close (structure/tag-expression structure) context)))
@@ -726,14 +743,25 @@ differences:
 	`(DEFINE (,name ,@lambda-list)
 	   ,(generate-body tag-expression)))))
 
-(define (default-value-expr structure name)
-  (let ((context (structure/context structure)))
-    `(,(absolute (if (structure/record-type? structure)
-		     'RECORD-TYPE-DEFAULT-VALUE
-		     'DEFINE-STRUCTURE/DEFAULT-VALUE)
-		 context)
-      ,(close (structure/type-descriptor structure) context)
-      ',name)))
+(define (default-value-expr structure slot)
+  (let ((expression (slot/default slot)))
+    ;; Scheme spends a remarkable amount of time fetching and calling
+    ;; the default value procedures.  This hack eliminates that time
+    ;; for many uses of DEFINE-STRUCTURE.
+    (if (not (pair? (strip-syntactic-closures expression)))
+	expression
+	(let ((record? (structure/record-type? structure))
+	      (context (structure/context structure)))
+	  `(,(absolute (if record?
+			   'RECORD-TYPE-DEFAULT-VALUE-BY-INDEX
+			   'DEFINE-STRUCTURE/DEFAULT-VALUE-BY-INDEX)
+		       context)
+	    ,(close (structure/type-descriptor structure) context)
+	    ,(- (slot/index slot)
+		(if record?
+		    0
+		    (+ (structure/offset structure)
+		       (if (structure/tagged? structure) 1 0)))))))))
 
 (define (copier-definitions structure)
   (let ((copier-name (structure/copier structure)))
@@ -754,15 +782,18 @@ differences:
 		(close (structure/tag-expression structure) context)))
 	  (case (structure/physical-type structure)
 	    ((RECORD)
-	     `((DEFINE ,predicate-name
-		 (LET ((TAG
-			(,(absolute 'RECORD-TYPE-DISPATCH-TAG context)
-			 ,tag-expression)))
-		   (NAMED-LAMBDA (,predicate-name OBJECT)
-		     (AND (,(absolute '%RECORD? context) OBJECT)
-			  (,(absolute 'EQ? context)
-			   (,(absolute '%RECORD-REF context) OBJECT 0)
-			   TAG)))))))
+	     (let ((tag-name (make-synthetic-identifier 'TAG)))
+	       `((DEFINE ,tag-name
+		   (,(absolute 'RECORD-TYPE-DISPATCH-TAG context)
+		    ,tag-expression))
+		 (DEFINE (,predicate-name OBJECT)
+		   (DECLARE
+		    (IGNORE-REFERENCE-TRAPS (SET ,(close tag-name context))))
+		   (AND (,(absolute '%RECORD? context) OBJECT)
+			(,(absolute 'EQ? context)
+			 (,(absolute '%RECORD-REF context) OBJECT 0)
+			 ;++ Work around a bug in the expander.
+			 ,(close tag-name context)))))))
 	    ((VECTOR)
 	     `((DEFINE (,predicate-name OBJECT)
 		 (AND (,(absolute 'VECTOR? context) OBJECT)
@@ -785,7 +816,8 @@ differences:
 	(tag-expression (structure/tag-expression structure))
 	(slots (structure/slots structure))
 	(context (structure/context structure))
-	(print-procedure (structure/print-procedure structure)))
+	(print-procedure (structure/print-procedure structure))
+	(print-entity-procedure (structure/print-entity-procedure structure)))
     (let ((name (symbol->string (parser-context/name context)))
 	  (field-names (map slot/name slots))
 	  (inits
@@ -798,7 +830,10 @@ differences:
 		 ',name
 		 ',field-names
 		 (LIST ,@inits)
-		 ,(close print-procedure context))
+		 ,(close print-procedure context)
+		 ,@(if print-entity-procedure
+		       (list (close print-entity-procedure context))
+		       '()))
 	       `(,(absolute 'MAKE-DEFINE-STRUCTURE-TYPE context)
 		 ',(structure/physical-type structure)
 		 ',name
@@ -814,7 +849,11 @@ differences:
 		      '#F)
 		 ',(+ (if (structure/tagged? structure) 1 0)
 		      (structure/offset structure)
-		      (length slots)))))
+		      (length slots))
+		 ,@(if (and (structure/tagged? structure)
+			    print-entity-procedure)
+		       (list (close print-entity-procedure context))
+		       '()))))
 	,@(if (and tag-expression
 		   (not (eq? tag-expression type-name)))
 	      `((,(absolute 'NAMED-STRUCTURE/SET-TAG-DESCRIPTION! context)
