@@ -1,9 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: global.scm,v 14.72 2004/11/19 07:14:30 cph Exp $
-
-Copyright 1988,1989,1991,1992,1993,1995 Massachusetts Institute of Technology
-Copyright 1998,2000,2001,2003,2004 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -19,7 +19,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -68,13 +68,17 @@ USA.
   system-vector?
   (system-vector-length system-vector-size)
   system-vector-ref
-  system-vector-set!)
+  system-vector-set!
+
+  primitive-object-ref primitive-object-set!)
 
 (define (host-big-endian?)
   host-big-endian?-saved)
 
 (define host-big-endian?-saved)
 
+(define ephemeron-type)
+
 (define (initialize-package!)
   ;; Assumptions:
   ;; * Word length is 32 or 64 bits.
@@ -89,11 +93,16 @@ USA.
 	  ((#x00010200 #x0001020000030400) #t)
 	  ((#x00020100 #x0004030000020100) #f)
 	  (else (error "Unable to determine endianness of host."))))
+  (add-secondary-gc-daemon! clean-obarray)
+  ;; Kludge until the next released version, to avoid a bootstrapping
+  ;; failure.
+  (set! ephemeron-type (microcode-type 'EPHEMERON))
   unspecific)
 
 ;;;; Potpourri
 
 (define (identity-procedure x) x)
+(define (constant-procedure k) (lambda args (declare (ignore args)) k))
 (define (null-procedure . args) args '())
 (define (false-procedure . args) args #f)
 (define (true-procedure . args) args #t)
@@ -116,36 +125,16 @@ USA.
 (define (limit-interrupts! limit-mask)
   (set-interrupt-enables! (fix:and limit-mask (get-interrupt-enables))))
 
-(define (object-component-binder get-component set-component!)
-  (lambda (object new-value thunk)
-    (let ((old-value))
-      (shallow-fluid-bind
-       (lambda ()
-	 (set! old-value (get-component object))
-	 (set-component! object new-value)
-	 (set! new-value #f)
-	 unspecific)
-       thunk
-       (lambda ()
-	 (set! new-value (get-component object))
-	 (set-component! object old-value)
-	 (set! old-value #f)
-	 unspecific)))))
+(define-integrable (object-component-binder get-component set-component!)
+  (lambda (object value thunk)
+    (define (swap!)
+      (let ((value* value))
+	(set! value (get-component object))
+	(set-component! object value*)))
+    (shallow-fluid-bind swap! thunk swap!)))
 
-(define (bind-cell-contents! cell new-value thunk)
-  (let ((old-value))
-    (shallow-fluid-bind
-     (lambda ()
-       (set! old-value (cell-contents cell))
-       (set-cell-contents! cell new-value)
-       (set! new-value)
-       unspecific)
-     thunk
-     (lambda ()
-       (set! new-value (cell-contents cell))
-       (set-cell-contents! cell old-value)
-       (set! old-value)
-       unspecific))))
+(define bind-cell-contents!
+  (object-component-binder cell-contents set-cell-contents!))
 
 (define (values . objects)
   (lambda (receiver)
@@ -197,15 +186,16 @@ USA.
 	     (process-time (- process-end process-start))
 	     (process-time/nogc
 	      (round->exact (* 1000 (- process-end/nogc process-start/nogc)))))
-	(write-string "process time: ")
-	(write process-time)
-	(write-string " (")
-	(write process-time/nogc)
-	(write-string " RUN + ")
-	(write (- process-time process-time/nogc))
-	(write-string " GC); real time: ")
-	(write (- real-end real-start))
-	(newline))
+	(write-notification-line
+	 (lambda (port)
+	   (write-string "process time: " port)
+	   (write process-time port)
+	   (write-string " (" port)
+	   (write process-time/nogc port)
+	   (write-string " RUN + " port)
+	   (write (- process-time process-time/nogc) port)
+	   (write-string " GC); real time: " port)
+	   (write (- real-end real-start) port))))
       value)))
 
 (define (wait-interval ticks)
@@ -265,7 +255,7 @@ USA.
   (%encode-gc-type ((ucode-primitive object-gc-type 1) object)))
 
 (define (type-code->gc-type code)
-  (%encode-gc-type ((ucode-primitive type-code->gc-type 1) code)))
+  (%encode-gc-type ((ucode-primitive type->gc-type 1) code)))
 
 (define (%encode-gc-type t)
   (if (not (and (fix:fixnum? t)
@@ -281,7 +271,6 @@ USA.
     ((NON-POINTER) #t)
     ((GC-INTERNAL)
      (or (object-type? (ucode-type manifest-nm-vector) object)
-	 (object-type? (ucode-type manifest-special-nm-vector) object)
 	 (and (object-type? (ucode-type reference-trap) object)
 	      (<= (object-datum object) trap-max-immediate))))
     (else #f)))
@@ -298,9 +287,7 @@ USA.
 (define (non-pointer-type-code? code)
   (case (type-code->gc-type code)
     ((NON-POINTER) #t)
-    ((GC-INTERNAL)
-     (or (fix:= (ucode-type manifest-nm-vector) code)
-	 (fix:= (ucode-type manifest-special-nm-vector) code)))
+    ((GC-INTERNAL) (fix:= (ucode-type manifest-nm-vector) code))
     (else #f)))
 
 (define (pointer-type-code? code)
@@ -313,68 +300,99 @@ USA.
   ;; Note: the unparser takes advantage of the fact that objects
   ;; satisfying this predicate also satisfy:
   ;; (object-type? (ucode-type constant) object)
-  (or (eq? object undefined-conditional-branch)
-      ;; same as `undefined-conditional-branch'.
-      ;; (eq? object *the-non-printing-object*)
-      ;; (eq? object unspecific)
+  (or (eq? object unspecific)
       (eq? object (object-new-type (ucode-type constant) 2))))
 
 (define unspecific
   (object-new-type (ucode-type constant) 1))
 
-(define (obarray->list #!optional obarray)
-  (let ((obarray
-	 (if (default-object? obarray)
-	     (fixed-objects-item 'OBARRAY)
-	     obarray)))
-    (let per-bucket
-	((index (fix:- (vector-length obarray) 1))
-	 (accumulator '()))
-      (if (fix:< index 0)
-	  accumulator
-	  (let per-symbol
-	      ((bucket (vector-ref obarray index))
-	       (accumulator accumulator))
-	    (if (pair? bucket)
-		(per-symbol (cdr bucket) (cons (car bucket) accumulator))
-		(per-bucket (fix:- index 1) accumulator)))))))
+(define (for-each-interned-symbol procedure)
+  (for-each-symbol-in-obarray (fixed-objects-item 'OBARRAY) procedure))
 
+(define (for-each-symbol-in-obarray obarray procedure)
+  (let per-bucket ((index (vector-length obarray)))
+    (if (fix:> index 0)
+	(let ((index (fix:- index 1)))
+	  (let per-symbol ((bucket (vector-ref obarray index)))
+	    (cond ((weak-pair? bucket)
+		   (let ((symbol (weak-car bucket)))
+		     (if (weak-pair/car? bucket)
+			 (procedure symbol)))
+		   (per-symbol (weak-cdr bucket)))
+		  ((pair? bucket)
+		   (procedure (car bucket))
+		   (per-symbol (cdr bucket)))
+		  (else
+		   (per-bucket index))))))))
+
+(define (obarray->list #!optional obarray)
+  (let ((list '()))
+    (define (accumulate symbol)
+      (set! list (cons symbol list))
+      unspecific)
+    (if (default-object? obarray)
+	(for-each-interned-symbol accumulate)
+	(for-each-symbol-in-obarray obarray accumulate))
+    list))
+
+(define (clean-obarray)
+  (without-interrupts
+   (lambda ()
+     (let ((obarray (fixed-objects-item 'OBARRAY)))
+       (let loop ((index (vector-length obarray)))
+	 (if (fix:> index 0)
+	     (let ((index (fix:- index 1)))
+	       (define (find-broken-entry bucket previous)
+		 (cond ((weak-pair? bucket)
+			(let ((d (weak-cdr bucket)))
+			  (if (weak-pair/car? bucket)
+			      (find-broken-entry d bucket)
+			      (delete-broken-entries d previous))))
+		       ((pair? bucket)
+			(find-broken-entry (cdr bucket) bucket))))
+	       (define (delete-broken-entries bucket previous)
+		 (cond ((weak-pair? bucket)
+			(let ((d (weak-cdr bucket)))
+			  (if (weak-pair/car? bucket)
+			      (begin (clobber previous bucket)
+				     (find-broken-entry d bucket))
+			      (delete-broken-entries d previous))))
+		       ((pair? bucket)
+			(clobber previous bucket)
+			(find-broken-entry (cdr bucket) bucket))
+		       (else
+			(clobber previous '()))))
+	       (define (clobber previous tail)
+		 (cond ((weak-pair? previous) (weak-set-cdr! previous tail))
+		       ((pair? previous) (set-cdr! previous tail))
+		       (else (vector-set! obarray index tail))))
+	       (find-broken-entry (vector-ref obarray index) #f)
+	       (loop index))))))))
+
 (define (impurify object)
-  (if (and (object-pointer? object) (object-pure? object))
-      ((ucode-primitive primitive-impurify) object))
   object)
 
-(define (fasdump object filename
-		 #!optional suppress-messages? dump-option)
-  (let* ((filename (->namestring (merge-pathnames filename)))
-	 (do-it
-	  (lambda (start-message end-message)
-	    (start-message)
-	    (let loop ()
-	      (if ((ucode-primitive primitive-fasdump)
-		   object filename
-		   (if (default-object? dump-option)
-		       #f
-		       dump-option))
-		  (end-message)
-		  (begin
-		    (with-simple-restart 'RETRY "Try again."
-		      (lambda ()
-			(error "FASDUMP: Object is too large to be dumped:"
-			       object)))
-		    (loop))))))
-	 (no-print (lambda () unspecific)))
-    (if (or (default-object? suppress-messages?)
-	    (not suppress-messages?))
-	(let ((port (notification-output-port)))
-	  (do-it (lambda ()
-		   (fresh-line port)
-		   (write-string ";Dumping " port)
-		   (write (enough-namestring filename) port))
-		 (lambda ()
-		   (write-string " -- done" port)
-		   (newline port))))
-	(do-it no-print no-print))))
+(define (fasdump object filename #!optional quiet? dump-option)
+  (let ((filename (->namestring (merge-pathnames filename)))
+	(quiet? (if (default-object? quiet?) #f quiet?))
+	(dump-option (if (default-object? dump-option) #f dump-option)))
+    (let ((do-it
+	   (lambda ()
+	     (let loop ()
+	       (if (not ((ucode-primitive primitive-fasdump)
+			 object filename dump-option))
+		   (begin
+		     (with-simple-restart 'RETRY "Try again."
+		       (lambda ()
+			 (error "FASDUMP: Object is too large to be dumped:"
+				object)))
+		     (loop)))))))
+    (if quiet?
+	(do-it)
+	(with-notification (lambda (port)
+			     (write-string "Dumping " port)
+			     (write (enough-namestring filename) port))
+	  do-it)))))
 
 ;;;; Hook lists
 
@@ -431,4 +449,62 @@ USA.
   (guarantee-hook-list hook-list 'RUN-HOOKS-IN-LIST)
   (for-each (lambda (p)
 	      (apply (cdr p) arguments))
-	    (hook-list-hooks hook-list)))
\ No newline at end of file
+	    (hook-list-hooks hook-list)))
+
+;;;; Ephemerons
+
+;;; The layout of an ephemeron is as follows:
+;;;
+;;;   0 vector (marked or non-marked) manifest
+;;;   1 key
+;;;   2 datum
+;;;   3 extra
+;;;   .  slots
+;;;   .    for
+;;;   .      GC
+
+(define canonical-false (list 'FALSE))
+
+(define (canonicalize object)
+  (if (eq? object #f)
+      canonical-false
+      object))
+
+(define (decanonicalize object)
+  (if (eq? object canonical-false)
+      #f
+      object))
+
+(define (make-ephemeron key datum)
+  ((ucode-primitive MAKE-EPHEMERON 2) (canonicalize key) (canonicalize datum)))
+
+(define (ephemeron? object)
+  (object-type? ephemeron-type object))
+
+(define-guarantee ephemeron "ephemeron")
+
+(define (ephemeron-key ephemeron)
+  (guarantee-ephemeron ephemeron 'EPHEMERON-KEY)
+  (decanonicalize (primitive-object-ref ephemeron 1)))
+
+(define (ephemeron-datum ephemeron)
+  (guarantee-ephemeron ephemeron 'EPHEMERON-DATUM)
+  (decanonicalize (primitive-object-ref ephemeron 2)))
+
+(define (set-ephemeron-key! ephemeron key)
+  (guarantee-ephemeron ephemeron 'SET-EPHEMERON-KEY!)
+  (let ((key* (primitive-object-ref ephemeron 1)))
+    (if key* (primitive-object-set! ephemeron 1 (canonicalize key)))
+    (reference-barrier key*))
+  unspecific)
+
+(define (set-ephemeron-datum! ephemeron datum)
+  (guarantee-ephemeron ephemeron 'SET-EPHEMERON-DATUM!)
+  (let ((key (primitive-object-ref ephemeron 1)))
+    (if key (primitive-object-set! ephemeron 2 (canonicalize datum)))
+    (reference-barrier key))
+  unspecific)
+
+(define (ephemeron-broken? ephemeron)
+  (guarantee-ephemeron ephemeron 'EPHEMERON-BROKEN?)
+  (not (primitive-object-ref ephemeron 1)))
\ No newline at end of file
