@@ -1,9 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: os2graph.scm,v 1.25 2003/11/10 21:46:16 cph Exp $
-
-Copyright 1995,1996,1997,1999,2000 Massachusetts Institute of Technology
-Copyright 2001,2002,2003 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -19,7 +19,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -206,7 +206,7 @@ USA.
 ;;;; Standard Operations
 
 (define (os2-graphics/available?)
-  (implemented-primitive-procedure? os2win-open))
+  (implemented-primitive-procedure? (ucode-primitive os2win-open 2)))
 
 (define (os2-graphics/open descriptor->device #!optional width height)
   (if (not event-descriptor)
