@@ -1,23 +1,27 @@
 #| -*-Scheme-*-
 
-$Id: os2winp.scm,v 1.18 2002/02/03 03:38:56 cph Exp $
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
-Copyright (c) 1995-1999, 2001, 2002 Massachusetts Institute of Technology
+This file is part of MIT/GNU Scheme.
 
-This program is free software; you can redistribute it and/or modify
+MIT/GNU Scheme is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or (at
 your option) any later version.
 
-This program is distributed in the hope that it will be useful, but
+MIT/GNU Scheme is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with MIT/GNU Scheme; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
+USA.
+
 |#
 
 ;;;; OS/2 PM Interface -- Primitives
@@ -416,7 +420,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 (define-integrable ROP_DSTINVERT   #x55)
 (define-integrable ROP_ZERO        #x00)
 (define-integrable ROP_ONE         #xFF)
-   
+
 ;; Constants for OS2PS-BITBLT options argument
 (define-integrable BBO_OR            0)
 (define-integrable BBO_AND           1)
