@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -605,7 +606,7 @@ these rules:
 (define (system-library-directory-pathname #!optional pathname required?)
   (if (if (default-object? pathname) #f pathname)
       (let ((dir (%find-library-file pathname)))
-	(cond ((file-directory? dir)
+	(cond ((and dir (file-directory? dir))
 	       (pathname-as-directory dir))
 	      ((if (default-object? required?) #f required?)
 	       (system-library-directory-pathname
