@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -77,14 +78,14 @@ USA.
 	 (error "Ill-formed regular s-expression:" regsexp))))
 
 (define (%compile-char-set items)
-  (scalar-values->alphabet
+  (scalar-values->char-set
    (append-map (lambda (item)
 		 (cond ((well-formed-scalar-value-range? item)
 			(list item))
 		       ((unicode-char? item)
 			(list (char->integer item)))
-		       ((alphabet? item)
-			(alphabet->scalar-values item))
+		       ((char-set? item)
+			(char-set->scalar-values item))
 		       ((string? item)
 			(map char->integer (string->list item)))
 		       (else
@@ -307,21 +308,21 @@ USA.
 			   (fail)))
 		     (succeed position groups fail)))))))))
 
-(define (insn:char-set alphabet)
+(define (insn:char-set char-set)
   (lambda (succeed)
     (lambda (position groups fail)
       (if (let ((char (next-char position)))
 	    (and char
-		 (char-in-alphabet? char alphabet)))
+		 (char-set-member? char-set char)))
 	  (succeed (next-position position) groups fail)
 	  (fail)))))
 
-(define (insn:inverse-char-set alphabet)
+(define (insn:inverse-char-set char-set)
   (lambda (succeed)
     (lambda (position groups fail)
       (if (let ((char (next-char position)))
 	    (and char
-		 (not (char-in-alphabet? char alphabet))))
+		 (not (char-set-member? char-set char))))
 	  (succeed (next-position position) groups fail)
 	  (fail)))))
 
