@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -95,6 +96,7 @@ USA.
 ;;;; Variable
 
 (define (make-variable name)
+  (guarantee-symbol name 'MAKE-VARIABLE)
   (system-hunk3-cons (ucode-type variable) name #t '()))
 
 (define (variable? object)
@@ -112,6 +114,7 @@ USA.
 ;;;; Definition/Assignment
 
 (define (make-definition name value)
+  (guarantee-symbol name 'MAKE-DEFINITION)
   (&typed-pair-cons (ucode-type definition) name value))
 
 (define (definition? object)
@@ -137,6 +140,7 @@ USA.
 (define-guarantee assignment "SCode assignment")
 
 (define (make-assignment-from-variable variable value)
+  (guarantee-variable variable 'MAKE-ASSIGNMENT-FROM-VARIABLE)
   (&typed-pair-cons (ucode-type assignment) variable value))
 
 (define (assignment-variable assignment)
@@ -152,6 +156,7 @@ USA.
 	    (assignment-value assignment)))
 
 (define (make-assignment name value)
+  (guarantee-symbol name 'MAKE-ASSIGNMENT)
   (make-assignment-from-variable (make-variable name) value))
 
 (define (assignment-name assignment)
@@ -238,6 +243,7 @@ USA.
 ;;;; Access
 
 (define (make-access environment name)
+  (guarantee-symbol name 'MAKE-ACCESS)
   (&typed-pair-cons (ucode-type access) environment name))
 
 (define (access? object)
