@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -48,6 +49,10 @@ USA.
 	     ((DATUM EXPRESSION FORM) #t)
 	     ((R4RS-BVL) (r4rs-lambda-list? object))
 	     ((MIT-BVL) (mit-lambda-list? object))
+	     ((STRING) (string? object))
+	     ((CHAR) (char? object))
+	     ((URI) (->uri object #f))
+	     ((INDEX) (exact-nonnegative-integer? object))
 	     (else (match-error))))
 	  ((pair? pattern)
 	   (case (car pattern)
