@@ -1,8 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: url.scm,v 1.36 2005/09/08 18:51:30 cph Exp $
-
-Copyright 2000,2001,2003,2004,2005 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,7 +19,7 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
@@ -30,20 +31,26 @@ USA.
 
 (declare (usual-integrations))
 
-(define-record-type <uri>
-    (%make-uri scheme authority path query fragment)
-    uri?
-  (scheme uri-scheme set-uri-scheme!)
-  (authority uri-authority)
-  (path uri-path)
-  (query uri-query)
-  (fragment uri-fragment set-uri-fragment!))
+(define-structure (uri
+		   (type-descriptor <uri>)
+		   (constructor %%make-uri)
+		   (conc-name %uri-)
+		   (print-procedure
+		    (simple-unparser-method 'URI
+		      (lambda (uri)
+			(list (uri->string uri))))))
+  (scheme #f read-only #t)
+  (authority #f read-only #t)
+  (path #f read-only #t)
+  (query #f read-only #t)
+  (fragment #f read-only #t)
+  (string #f read-only #t))
 
-(set-record-type-unparser-method! <uri>
-  (standard-unparser-method 'URI
-    (lambda (uri port)
-      (write-char #\space port)
-      (%write-uri uri port))))
+(define uri-parser-method
+  (simple-parser-method
+   (lambda (objects)
+     (and (pair? objects)
+	  (string->uri (car objects))))))
 
 (define (make-uri scheme authority path query fragment)
   (let ((path (if (equal? path '("")) '() path)))
@@ -54,13 +61,40 @@ USA.
     (if fragment (guarantee-utf8-string fragment 'MAKE-URI))
     (if (and authority (pair? path) (path-relative? path))
 	(error:bad-range-argument path 'MAKE-URI))
-    (%make-uri scheme
-	       authority
-	       (if scheme
-		   (remove-dot-segments path)
-		   path)
-	       query
-	       fragment)))
+    (%make-uri scheme authority path query fragment)))
+
+(define (%make-uri scheme authority path query fragment)
+  (let* ((path (remove-dot-segments path))
+	 (string
+	  (call-with-output-string
+	    (lambda (port)
+	      (%write-uri scheme authority path query fragment port)))))
+    (hash-table/intern! interned-uris string
+      (lambda ()
+	(%%make-uri scheme authority path query fragment string)))))
+
+(define interned-uris)
+
+(define (uri-scheme uri)
+  (%uri-scheme (->uri uri 'URI-SCHEME)))
+
+(define (uri-authority uri)
+  (%uri-authority (->uri uri 'URI-AUTHORITY)))
+
+(define (uri-path uri)
+  (%uri-path (->uri uri 'URI-PATH)))
+
+(define (uri-query uri)
+  (%uri-query (->uri uri 'URI-QUERY)))
+
+(define (uri-fragment uri)
+  (%uri-fragment (->uri uri 'URI-FRAGMENT)))
+
+(define (uri-absolute? uri)
+  (if (uri-scheme uri) #t #f))
+
+(define (uri-relative? uri)
+  (if (uri-scheme uri) #f #t))
 
 (define (absolute-uri? object)
   (and (uri? object)
@@ -70,19 +104,12 @@ USA.
   (and (uri? object)
        (uri-relative? object)))
 
-(define-integrable (uri-absolute? uri)
-  (if (uri-scheme uri) #t #f))
-
-(define-integrable (uri-relative? uri)
-  (if (uri-scheme uri) #f #t))
-
-(define-guarantee uri "URI")
-(define-guarantee relative-uri "relative URI")
-(define-guarantee absolute-uri "absolute URI")
+(define (error:not-uri object caller)
+  (error:wrong-type-argument object "URI" caller))
 
 (define (uri-scheme? object)
   (and (interned-symbol? object)
-       (complete-match matcher:scheme (symbol-name object))))
+       (*match-symbol matcher:scheme object)))
 
 ;;; A well-formed path is a list of N segments which is equivalent to
 ;;; a string in which there is a slash between each adjacent pair of
@@ -109,18 +136,19 @@ USA.
 (define-integrable (path-relative? path)
   (not (path-absolute? path)))
 
-(define-record-type <uri-authority>
-    (%make-uri-authority userinfo host port)
-    uri-authority?
-  (userinfo uri-authority-userinfo)
-  (host uri-authority-host)
-  (port uri-authority-port))
-
-(set-record-type-unparser-method! <uri-authority>
-  (standard-unparser-method 'URI-AUTHORITY
-    (lambda (authority port)
-      (write-char #\space port)
-      (write-authority authority port))))
+(define-structure (uri-authority
+		   (type-descriptor <uri-authority>)
+		   (constructor %%make-uri-authority)
+		   (conc-name %uri-authority-)
+		   (print-procedure
+		    (simple-unparser-method 'URI-AUTHORITY
+		      (lambda (authority)
+			(list (call-with-output-string
+				(lambda (port)
+				  (write-uri-authority authority port))))))))
+  (userinfo #f read-only #t)
+  (host #f read-only #t)
+  (port #f read-only #t))
 
 (define (make-uri-authority userinfo host port)
   (if userinfo (guarantee-uri-userinfo userinfo 'MAKE-URI-AUTHORITY))
@@ -128,13 +156,33 @@ USA.
   (if port (guarantee-uri-port port 'MAKE-URI-AUTHORITY))
   (%make-uri-authority userinfo host port))
 
+(define (%make-uri-authority userinfo host port)
+  (hash-table/intern! interned-uri-authorities
+      (call-with-output-string
+	(lambda (output)
+	  (%write-authority userinfo host port output)))
+    (lambda ()
+      (%%make-uri-authority userinfo host port))))
+
+(define interned-uri-authorities)
+
+(define (uri-authority-userinfo authority)
+  (guarantee-uri-authority authority 'URI-AUTHORITY-USERINFO)
+  (%uri-authority-userinfo authority))
+
+(define (uri-authority-host authority)
+  (guarantee-uri-authority authority 'URI-AUTHORITY-HOST)
+  (%uri-authority-host authority))
+
+(define (uri-authority-port authority)
+  (guarantee-uri-authority authority 'URI-AUTHORITY-PORT)
+  (%uri-authority-port authority))
+
 (define (uri-userinfo? object)
-  (and (string? object)
-       (complete-match parser:userinfo object)))
+  (utf8-string? object))
 
 (define (uri-host? object)
-  (and (string? object)
-       (complete-match matcher:host object)))
+  (utf8-string? object))
 
 (define (uri-port? object)
   (exact-nonnegative-integer? object))
@@ -146,103 +194,78 @@ USA.
 (define-guarantee uri-host "URI host")
 (define-guarantee uri-port "URI port")
 
-(define (complete-match matcher string #!optional start end)
-  (let ((buffer (string->parser-buffer string start end)))
-    (and (matcher buffer)
-	 (not (peek-parser-buffer-char buffer)))))
-
 (define (uri=? u1 u2)
-  (let ((u1 (->uri u1 'URI=?))
-	(u2 (->uri u2 'URI=?)))
-    (and (eq? (uri-scheme u1) (uri-scheme u2))
-	 (%component=? %uri-authority=? (uri-authority u1) (uri-authority u2))
-	 (let loop ((p1 (uri-path u1)) (p2 (uri-path u2)))
-	   (if (pair? p1)
-	       (and (pair? p2)
-		    (string=? (car p1) (car p2))
-		    (loop (cdr p1) (cdr p2)))
-	       (null? p2)))
-	 (%component=? string=? (uri-query u1) (uri-query u2))
-	 (%component=? string=? (uri-fragment u1) (uri-fragment u2)))))
+  (eq? (->uri u1 'URI=?)
+       (->uri u2 'URI=?)))
 
 (define (uri-authority=? a1 a2)
   (guarantee-uri-authority a1 'URI-AUTHORITY=?)
   (guarantee-uri-authority a2 'URI-AUTHORITY=?)
-  (%uri-authority=? a1 a2))
-
-(define (%uri-authority=? a1 a2)
-  (and (%component=? string=?
-		     (uri-authority-userinfo a1)
-		     (uri-authority-userinfo a2))
-       (string=? (uri-authority-host a1) (uri-authority-host a2))
-       (%component=? = (uri-authority-port a1) (uri-authority-port a2))))
-
-(define (%component=? predicate x1 x2)
-  (if x1
-      (and x2 (predicate x1 x2))
-      (not x2)))
+  (eq? a1 a2))
 
 (define (uri->alist uri)
-  `(,@(if (uri-scheme uri)
-	  `((scheme ,(uri-scheme uri)))
-	  '())
-    ,@(if (uri-authority uri)
-	  (let ((a (uri-authority uri)))
-	    `(,@(if (uri-authority-userinfo a)
-		    `((userinfo ,(uri-authority-userinfo a)))
-		    '())
-	      (host ,(uri-authority-host a))
-	      ,@(if (uri-authority-port a)
-		    `((port ,(uri-authority-port a)))
-		    '())))
-	  '())
-    (path ,(uri-path uri))
-    ,@(if (uri-query uri)
-	  `((query ,(uri-query uri)))
-	  '())
-    ,@(if (uri-fragment uri)
-	  `((fragment ,(uri-fragment uri)))
-	  '())))
+  (let ((uri (->uri uri 'URI->ALIST)))
+    `(,@(if (%uri-scheme uri)
+	    `((scheme ,(%uri-scheme uri)))
+	    '())
+      ,@(if (%uri-authority uri)
+	    (let ((a (%uri-authority uri)))
+	      `(,@(if (%uri-authority-userinfo a)
+		      `((userinfo ,(%uri-authority-userinfo a)))
+		      '())
+		(host ,(%uri-authority-host a))
+		,@(if (%uri-authority-port a)
+		      `((port ,(%uri-authority-port a)))
+		      '())))
+	    '())
+      (path ,(%uri-path uri))
+      ,@(if (%uri-query uri)
+	    `((query ,(%uri-query uri)))
+	    '())
+      ,@(if (%uri-fragment uri)
+	    `((fragment ,(%uri-fragment uri)))
+	    '()))))
+
+(define (uri-prefix prefix)
+  (guarantee-utf8-string prefix 'URI-PREFIX)
+  (lambda (suffix)
+    (guarantee-utf8-string suffix 'URI-PREFIX)
+    (string->absolute-uri (string-append prefix suffix))))
 
 ;;;; Merging
 
 (define (merge-uris uri base-uri)
-  (guarantee-absolute-uri base-uri 'MERGE-URIS)
-  (let ((uri (->uri uri 'MERGE-URIS)))
-    (cond ((uri-scheme uri)
-	   (%make-uri (uri-scheme uri)
-		      (uri-authority uri)
-		      (remove-dot-segments (uri-path uri))
-		      (uri-query uri)
-		      (uri-fragment uri)))
-	  ((uri-authority uri)
-	   (%make-uri (uri-scheme base-uri)
-		      (uri-authority uri)
-		      (remove-dot-segments (uri-path uri))
-		      (uri-query uri)
-		      (uri-fragment uri)))
-	  ((null? (uri-path uri))
-	   (%make-uri (uri-scheme base-uri)
-		      (uri-authority base-uri)
-		      (uri-path base-uri)
-		      (or (uri-query uri) (uri-query base-uri))
-		      (uri-fragment uri)))
+  (let ((uri (->uri uri 'MERGE-URIS))
+	(base-uri (->absolute-uri base-uri 'MERGE-URIS)))
+    (cond ((%uri-scheme uri)
+	   uri)
+	  ((%uri-authority uri)
+	   (%make-uri (%uri-scheme base-uri)
+		      (%uri-authority uri)
+		      (%uri-path uri)
+		      (%uri-query uri)
+		      (%uri-fragment uri)))
+	  ((null? (%uri-path uri))
+	   (%make-uri (%uri-scheme base-uri)
+		      (%uri-authority base-uri)
+		      (%uri-path base-uri)
+		      (or (%uri-query uri) (%uri-query base-uri))
+		      (%uri-fragment uri)))
 	  (else
-	   (%make-uri (uri-scheme base-uri)
-		      (uri-authority base-uri)
-		      (remove-dot-segments
-		       (merge-paths (uri-path uri) base-uri))
-		      (uri-query uri)
-		      (uri-fragment uri))))))
+	   (%make-uri (%uri-scheme base-uri)
+		      (%uri-authority base-uri)
+		      (merge-paths (%uri-path uri) base-uri)
+		      (%uri-query uri)
+		      (%uri-fragment uri))))))
 
 (define (merge-paths ref-path base-uri)
   (cond ((path-absolute? ref-path)
 	 ref-path)
-	((and (uri-authority base-uri)
-	      (null? (uri-path base-uri)))
+	((and (%uri-authority base-uri)
+	      (null? (%uri-path base-uri)))
 	 (cons "" ref-path))
 	(else
-	 (let ((path (uri-path base-uri)))
+	 (let ((path (%uri-path base-uri)))
 	   (if (and (pair? path)
 		    (pair? (cdr path)))
 	       (append (except-last-pair path) ref-path)
@@ -288,37 +311,53 @@ USA.
 	  (if (pair? input)
 	      (some-output input output)
 	      (cons "" output)))))
-    (reverse! (no-output path))))
+    (if (path-absolute? path)
+	(reverse! (no-output path))
+	path)))
 
 ;;;; Parser
 
-(define-syntax define-uri-coercion
-  (sc-macro-transformer
-   (lambda (form environment)
-     environment
-     (if (syntax-match? '(SYMBOL) (cdr form))
-	 (let* ((root (cadr form)))
-	   `(DEFINE (,(symbol '-> root) OBJECT #!OPTIONAL CALLER)
-	      (IF (,(symbol root '?) OBJECT)
-		  OBJECT
-		  (OR (COMPLETE-PARSE
-		       ,(symbol 'PARSE- root)
-		       (OR (->PARSER-BUFFER OBJECT)
-			   (,(symbol 'ERROR:NOT- root) OBJECT CALLER)))
-		      (ERROR:BAD-RANGE-ARGUMENT OBJECT CALLER)))))
-	 (ill-formed-syntax form)))))
+(define (->uri object #!optional caller)
+  (%->uri object parse-uri (lambda (uri) uri #t) caller))
 
-(define-uri-coercion uri)
-(define-uri-coercion absolute-uri)
-(define-uri-coercion relative-uri)
+(define (->absolute-uri object #!optional caller)
+  (%->uri object parse-absolute-uri uri-absolute? caller))
 
-(define (->parser-buffer object)
-  (cond ((or (string? object) (wide-string? object))
-	 (string->parser-buffer object))
-	((input-port? object) (input-port->parser-buffer object))
-	((symbol? object) (string->parser-buffer (symbol->wide-string object)))
-	(else #f)))
+(define (->relative-uri object #!optional caller)
+  (%->uri object parse-relative-uri uri-relative? caller))
 
+(define (%->uri object parser predicate caller)
+  ;; Kludge: take advantage of fact that (NOT (NOT #!DEFAULT)).
+  (let* ((do-parse
+	  (lambda (string)
+	    (let ((v (*parse-string parser string)))
+	      (if v
+		  (vector-ref v 0)
+		  (begin
+		    (if caller (error:bad-range-argument object caller))
+		    #f)))))
+	 (do-string
+	  (lambda (string)
+	    (or (hash-table/get interned-uris string #f)
+		(do-parse (utf8-string->wide-string string))))))
+    (cond ((uri? object)
+	   (if (predicate object)
+	       object
+	       (begin
+		 (if caller (error:bad-range-argument object caller))
+		 #f)))
+	  ((string? object)
+	   (do-string object))
+	  ((symbol? object)
+	   (do-string (symbol-name object)))
+	  ((wide-string? object)
+	   (let ((string (wide-string->utf8-string object)))
+	     (or (hash-table/get interned-uris string #f)
+		 (do-parse object))))
+	  (else
+	   (if caller (error:not-uri object caller))
+	   #f))))
+
 (define (string->uri string #!optional start end)
   (%string->uri parse-uri string start end 'STRING->URI))
 
@@ -329,23 +368,23 @@ USA.
   (%string->uri parse-relative-uri string start end 'STRING->RELATIVE-URI))
 
 (define (%string->uri parser string start end caller)
-  (or (complete-parse parser (string->parser-buffer string start end))
+  (or (and (string? string)
+	   (default-object? start)
+	   (default-object? end)
+	   (hash-table/get interned-uris string #f))
+      (let ((v (*parse-string parser string start end)))
+	(and v
+	     (vector-ref v 0)))
       (error:bad-range-argument string caller)))
 
-(define (complete-parse parser buffer)
-  (let ((v (parser buffer)))
-    (and v
-	 (not (peek-parser-buffer-char buffer))
-	 (vector-ref v 0))))
-
 (define parse-uri
-  (*parser (top-level (encapsulate encapsulate-uri parser:uri-reference))))
+  (*parser (encapsulate encapsulate-uri parser:uri-reference)))
 
 (define parse-absolute-uri
-  (*parser (top-level (encapsulate encapsulate-uri parser:uri))))
+  (*parser (encapsulate encapsulate-uri parser:uri)))
 
 (define parse-relative-uri
-  (*parser (top-level (encapsulate encapsulate-uri parser:relative-ref))))
+  (*parser (encapsulate encapsulate-uri parser:relative-ref)))
 
 (define (encapsulate-uri v)
   (%make-uri (vector-ref v 0)
@@ -353,6 +392,12 @@ USA.
 	     (vector-ref v 2)
 	     (vector-ref v 3)
 	     (vector-ref v 4)))
+
+(define parse-uri-path-absolute
+  (*parser
+   (encapsulate encapsulate-uri
+     (seq (values #f #f)
+	  parser:path-absolute))))
 
 (define parser:uri
   (*parser
@@ -366,10 +411,11 @@ USA.
 
 (define parser:hier-part
   (*parser
-   (alt (seq "//" parser:authority parser:path-abempty)
-	(seq (values #f) parser:path-absolute)
-	(seq (values #f) parser:path-rootless)
-	(seq (values #f) parser:path-empty))))
+   (alt (seq "//" parse-uri-authority parser:path-abempty)
+	(seq (values #f)
+	     (alt parser:path-absolute
+		  parser:path-rootless
+		  parser:path-empty)))))
 
 (define parser:uri-reference
   (*parser
@@ -387,10 +433,11 @@ USA.
 
 (define parser:relative-part
   (*parser
-   (alt (seq "//" parser:authority parser:path-abempty)
-	(seq (values #f) parser:path-absolute)
-	(seq (values #f) parser:path-noscheme)
-	(seq (values #f) parser:path-empty))))
+   (alt (seq "//" parse-uri-authority parser:path-abempty)
+	(seq (values #f)
+	     (alt parser:path-absolute
+		  parser:path-noscheme
+		  parser:path-empty)))))
 
 (define parser:scheme
   (*parser
@@ -401,7 +448,7 @@ USA.
    (seq (char-set char-set:uri-alpha)
 	(* (char-set char-set:uri-scheme)))))
 
-(define parser:authority
+(define parse-uri-authority
   (*parser
    (encapsulate (lambda (v)
 		  (%make-uri-authority (vector-ref v 0)
@@ -413,12 +460,15 @@ USA.
 
 (define parser:hostport
   (*parser
-   (seq (map uri-string-downcase (match matcher:host))
+   (seq (map uri-string-downcase
+	     (alt (match matcher:ip-literal)
+		  ;; subsumed by MATCHER:REG-NAME
+		  ;;matcher:ipv4-address
+		  (map decode-component
+		       (match matcher:reg-name))))
 	(alt (seq ":"
-		  (map (lambda (s)
-			 (and (fix:> (string-length s) 0)
-			      (string->number s)))
-		       (match (* (char-set char-set:uri-digit)))))
+		  (map string->number
+		       (match (+ (char-set char-set:uri-digit)))))
 	     (values #f)))))
 
 ;; This is a kludge to work around fact that STRING-DOWNCASE only
@@ -435,13 +485,6 @@ USA.
 		 (write-char (char-downcase char) output)
 		 (loop)))))))))
 
-(define matcher:host
-  (*matcher
-   (alt matcher:ip-literal
-	;; subsumed by MATCHER:REG-NAME
-	;;matcher:ipv4-address
-	matcher:reg-name)))
-
 (define matcher:ip-literal
   (*matcher
    (seq "["
@@ -539,64 +582,68 @@ USA.
 ;;;; Output
 
 (define (uri->string uri)
-  (guarantee-uri uri 'URI->STRING)
-  (call-with-output-string
-    (lambda (port)
-      (%write-uri uri port))))
+  (%uri-string (->uri uri 'URI->STRING)))
 
 (define (uri->symbol uri)
   (utf8-string->symbol (uri->string uri)))
 
 (define (write-uri uri port)
-  (guarantee-uri uri 'WRITE-URI)
-  (guarantee-port port 'WRITE-URI)
-  (%write-uri uri port))
+  (write-string (uri->string uri) port))
 
-(define (%write-uri uri port)
-  (if (uri-scheme uri)
+(define (%write-uri scheme authority path query fragment port)
+  (if scheme
       (begin
-	(write (uri-scheme uri) port)
+	(write scheme port)
 	(write-char #\: port)))
-  (if (uri-authority uri)
-      (write-authority (uri-authority uri) port))
-  (let ((path (uri-path uri)))
-    (if (pair? path)
-	(begin
-	  (if (uri-scheme uri)
-	      (write-segment (car path) port)
-	      (write-encoded (car path) char-set:uri-segment-nc port))
-	  (for-each (lambda (segment)
-		      (write-char #\/ port)
-		      (write-segment segment port))
-		    (cdr path)))))
-  (if (uri-query uri)
+  (if authority
+      (write-uri-authority authority port))
+  (if (pair? path)
+      (begin
+	(if scheme
+	    (write-segment (car path) port)
+	    (write-encoded (car path) char-set:uri-segment-nc port))
+	(for-each (lambda (segment)
+		    (write-char #\/ port)
+		    (write-segment segment port))
+		  (cdr path))))
+  (if query
       (begin
 	(write-char #\? port)
-	(write-encoded (uri-query uri) char-set:uri-query port)))
-  (if (uri-fragment uri)
+	(write-encoded query char-set:uri-query port)))
+  (if fragment
       (begin
 	(write-char #\# port)
-	(write-encoded (uri-fragment uri) char-set:uri-fragment port))))
+	(write-encoded fragment char-set:uri-fragment port))))
 
-(define (write-authority authority port)
-  (write-string "//" port)
-  (if (uri-authority-userinfo authority)
+(define (write-uri-authority authority port)
+  (%write-authority (%uri-authority-userinfo authority)
+		    (%uri-authority-host authority)
+		    (%uri-authority-port authority)
+		    port))
+
+(define (%write-authority userinfo host port output)
+  (write-string "//" output)
+  (if userinfo
       (begin
-	(write-encoded (uri-authority-userinfo authority)
-		       char-set:uri-userinfo
-		       port)
-	(write-char #\@ port)))
-  (if (uri-authority-host authority)
-      (write-encoded (uri-authority-host authority)
-		     char-set:uri-opaque-auth
-		     port))
-  (if (uri-authority-port authority)
+	(write-encoded userinfo char-set:uri-userinfo output)
+	(write-char #\@ output)))
+  (if host
+      (if (*match-string matcher:ip-literal host)
+	  (write-string host output)
+	  (write-encoded host char-set:uri-reg-name output)))
+  (if port
       (begin
-	(write-char #\: port)
-	(write (uri-authority-port authority) port))))
+	(write-char #\: output)
+	(write port output))))
 
 (define (write-segment segment port)
   (write-encoded segment char-set:uri-segment port))
+
+(define (encode-uri-path-segment segment)
+  (guarantee-string segment 'ENCODE-URI-PATH-SEGMENT)
+  (call-with-output-string
+    (lambda (port)
+      (write-segment segment port))))
 
 ;;;; Escape codecs
 
@@ -710,9 +757,6 @@ USA.
   (rexp-sequence char-set:uri-alpha
 		 (rexp* char-set:uri-scheme)))
 
-(define (uri-rexp:opaque-auth)
-  (rexp* char-set:uri-opaque-auth))
-
 (define (uri-rexp:authority)
   (rexp-sequence (rexp-optional (uri-rexp:userinfo) "@")
 		 (uri-rexp:host)
@@ -843,13 +887,13 @@ USA.
 (define char-set:uri-hex)
 (define char-set:uri-scheme)
 (define char-set:uri-userinfo)
-(define char-set:uri-opaque-auth)
 (define char-set:uri-ipvfuture)
 (define char-set:uri-reg-name)
 (define char-set:uri-segment)
 (define char-set:uri-segment-nc)
 (define char-set:uri-query)
 (define char-set:uri-fragment)
+(define char-set:uri-sloppy-auth)
 
 (define parser:userinfo)
 (define matcher:reg-name)
@@ -872,24 +916,22 @@ USA.
 	(char-set-union char-set:uri-alpha
 			char-set:uri-digit
 			(string->char-set "+-.")))
-  (let* ((uri-char
+  (let* ((sub-delims (string->char-set "!$&'()*+,;="))
+	 (unreserved
 	  (char-set-union char-set:uri-alpha
 			  char-set:uri-digit
-			  (string->char-set "!$&'()*+,-./:;=?@_~")))
+			  (string->char-set "-._~")))
 	 (component-chars
-	  (lambda (free)
-	    (char-set-difference uri-char (string->char-set free)))))
-    (set! char-set:uri-userinfo		(component-chars "/?@"))
+	  (lambda (extra)
+	    (char-set-union unreserved sub-delims (string->char-set extra)))))
+    (set! char-set:uri-userinfo		(component-chars ":"))
     (set! char-set:uri-ipvfuture	char-set:uri-userinfo)
-    (set! char-set:uri-reg-name		(component-chars "/:?@"))
-    (set! char-set:uri-segment		(component-chars "/?"))
-    (set! char-set:uri-segment-nc	(component-chars "/:?"))
-    (set! char-set:uri-query		uri-char)
-    (set! char-set:uri-fragment		uri-char)
-
-    (set! char-set:uri-opaque-auth
-	  (char-set-union (component-chars "/?")
-			  (string->char-set "[]"))))
+    (set! char-set:uri-reg-name		(component-chars ""))
+    (set! char-set:uri-segment		(component-chars ":@"))
+    (set! char-set:uri-segment-nc	(component-chars "@"))
+    (set! char-set:uri-query		(component-chars ":@/?"))
+    (set! char-set:uri-fragment		char-set:uri-query)
+    (set! char-set:uri-sloppy-auth	(component-chars ":@[]")))
 
   (set! parser:userinfo		(component-parser-* char-set:uri-userinfo))
   (set! matcher:reg-name	(component-matcher-* char-set:uri-reg-name))
@@ -899,6 +941,9 @@ USA.
   (set! parser:query		(component-parser-* char-set:uri-query))
   (set! parser:fragment		(component-parser-* char-set:uri-fragment))
 
+  (set! interned-uris (make-string-hash-table))
+  (set! interned-uri-authorities (make-string-hash-table))
+
   ;; backwards compatibility:
   (set! url:char-set:unreserved
 	(char-set-union char-set:uri-alpha
@@ -907,7 +952,7 @@ USA.
   (set! url:char-set:unescaped
 	(char-set-union url:char-set:unreserved
 			(string->char-set ";/?:@&=")))
-  unspecific)
+  (define-bracketed-object-parser-method 'URI uri-parser-method))
 
 ;;;; Testing
 
@@ -1202,7 +1247,7 @@ USA.
   (EOF))
 
 (define-ppu-state authority
-  (opaque-auth (push) authority)
+  (sloppy-auth (push) authority)
   (/ (set authority) (push) path)
   (? (set authority) query)
   (|#| (set authority) fragment)
