@@ -1,23 +1,27 @@
 #| -*-Scheme-*-
 
-$Id: vector.scm,v 14.21 2002/02/09 06:10:11 cph Exp $
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
-Copyright (c) 1988-2002 Massachusetts Institute of Technology
+This file is part of MIT/GNU Scheme.
 
-This program is free software; you can redistribute it and/or modify
+MIT/GNU Scheme is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or (at
 your option) any later version.
 
-This program is distributed in the hope that it will be useful, but
+MIT/GNU Scheme is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+along with MIT/GNU Scheme; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
+
 |#
 
 ;;;; Operations on Vectors
@@ -26,9 +30,18 @@ USA.
 (declare (usual-integrations))
 
 (define-primitives
-  vector? vector-length vector-ref vector-set!
-  list->vector vector subvector->list
-  subvector-move-right! subvector-move-left! subvector-fill!)
+  (list->vector 1)
+  (primitive-make-object 2)
+  (primitive-object-set! 3)
+  (subvector->list 3)
+  (subvector-fill! 4)
+  (subvector-move-left! 5)
+  (subvector-move-right! 5)
+  (vector -1)
+  (vector-length 1)
+  (vector-ref 2)
+  (vector-set! 3)
+  (vector? 1))
 
 (define-integrable (guarantee-vector object procedure)
   (if (not (vector? object))
@@ -67,6 +80,14 @@ USA.
 (define-integrable (vector-head vector end)
   (subvector vector 0 end))
 
+(define (vector-head! vector end)
+  (guarantee-subvector vector 0 end 'VECTOR-HEAD!)
+  (if (fix:< end (vector-length vector))
+      (primitive-object-set! vector 0
+			     (primitive-make-object (ucode-type false)
+						    end)))
+  vector)
+
 (define (vector-tail vector start)
   (guarantee-vector vector 'VECTOR-TAIL)
   (subvector vector start (vector-length vector)))
@@ -115,41 +136,40 @@ USA.
 	    (loop (fix:+ index 1)))))
     vector))
 
-(define (vector-map procedure vector)
-  (if (vector? procedure)
-      ;; KLUDGE: accept arguments in old order.
-      (vector-map vector procedure)
-      (begin
-	(guarantee-vector vector 'VECTOR-MAP)
-	(let ((length (vector-length vector)))
-	  (if (fix:= 0 length)
-	      vector
-	      (let ((result (make-vector length)))
-		(let loop ((index 0))
-		  (if (fix:< index length)
-		      (begin
-			(vector-set! result
-				     index
-				     (procedure (vector-ref vector index)))
-			(loop (fix:+ index 1)))))
-		result))))))
+(define (vector-map procedure vector . vectors)
+  (guarantee-vector vector 'VECTOR-MAP)
+  (for-each (lambda (v) (guarantee-vector v 'VECTOR-MAP)) vectors)
+  (let ((n (vector-length vector)))
+    (for-each (lambda (v)
+		(if (not (fix:= (vector-length v) n))
+		    (error:bad-range-argument v 'VECTOR-MAP)))
+	      vectors)
+    (let ((result (make-vector n)))
+      (do ((i 0 (fix:+ i 1)))
+	  ((not (fix:< i n)))
+	(vector-set! result
+		     i
+		     (apply procedure
+			    (vector-ref vector i)
+			    (map (lambda (v) (vector-ref v i)) vectors))))
+      result)))
+
+(define (vector-for-each procedure vector . vectors)
+  (guarantee-vector vector 'VECTOR-FOR-EACH)
+  (for-each (lambda (v) (guarantee-vector v 'VECTOR-FOR-EACH)) vectors)
+  (let ((n (vector-length vector)))
+    (for-each (lambda (v)
+		(if (not (fix:= (vector-length v) n))
+		    (error:bad-range-argument v 'VECTOR-FOR-EACH)))
+	      vectors)
+    (do ((i 0 (fix:+ i 1)))
+	((not (fix:< i n)) unspecific)
+      (apply procedure
+	     (vector-ref vector i)
+	     (map (lambda (v) (vector-ref v i)) vectors)))))
 
 (define (for-each-vector-element vector procedure)
-  (guarantee-vector vector 'FOR-EACH-VECTOR-ELEMENT)
-  (let ((length (vector-length vector)))
-    (let loop ((index 0))
-      (if (fix:< index length)
-	  (begin
-	    (procedure (vector-ref vector index))
-	    (loop (fix:+ index 1)))))))
-
-(define (vector-of-type? vector predicate)
-  (and (vector? vector)
-       (let ((n (vector-length vector)))
-	 (let loop ((i 0))
-	   (or (fix:= i n)
-	       (and (predicate (vector-ref vector i))
-		    (loop (fix:+ i 1))))))))
+  (vector-for-each procedure vector))
 
 (define (subvector-find-next-element vector start end item)
   (guarantee-subvector vector start end 'SUBVECTOR-FIND-NEXT-ELEMENT)
@@ -207,7 +227,7 @@ USA.
       (sc-macro-transformer
        (lambda (form environment)
 	 `(DEFINE-INTEGRABLE (,(cadr form) VECTOR)
-	    (GUARANTEE-VECTOR VECTOR 'SAFE-VECTOR-REF)
+	    (GUARANTEE-VECTOR VECTOR ',(cadr form))
 	    (VECTOR-REF VECTOR ,(caddr form)))))))
   (iref vector-first 0)
   (iref vector-second 1)
@@ -241,4 +261,34 @@ USA.
 
 (define (vector-uniform? vector)
   (guarantee-vector vector 'VECTOR-UNIFORM?)
-  (subvector-uniform? vector 0 (vector-length vector)))
\ No newline at end of file
+  (subvector-uniform? vector 0 (vector-length vector)))
+
+(define (vector-of-type? object predicate)
+  (and (vector? object)
+       (let ((n (vector-length object)))
+	 (let loop ((i 0))
+	   (if (fix:< i n)
+	       (and (predicate (vector-ref object i))
+		    (loop (fix:+ i 1)))
+	       #t)))))
+
+(define (guarantee-vector-of-type object predicate description caller)
+  (if (not (vector-of-type? object predicate))
+      (error:wrong-type-argument object description caller)))
+
+(define (vector-of-unique-symbols? object)
+  (and (vector? object)
+       (let ((n (vector-length object)))
+	 (let loop ((i 0))
+	   (if (fix:< i n)
+	       (let ((elt (vector-ref object i)))
+		 (and (symbol? elt)
+		      (let find-dup ((i (fix:+ i 1)))
+			(if (fix:< i n)
+			    (and (not (eq? (vector-ref object i) elt))
+				 (find-dup (fix:+ i 1)))
+			    #t))
+		      (loop (fix:+ i 1))))
+	       #t)))))
+
+(define-guarantee vector-of-unique-symbols "vector of unique symbols")
\ No newline at end of file
