@@ -1,8 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: load.scm,v 1.16 2003/04/25 03:53:58 cph Exp $
-
-Copyright 2001,2002,2003 Massachusetts Institute of Technology
+Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
+    1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -18,12 +19,12 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with MIT/GNU Scheme; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
 USA.
 
 |#
 
-(with-working-directory-pathname (directory-pathname (current-load-pathname))
+(with-loader-base-uri (system-library-uri "star-parser/")
   (lambda ()
-    (package/system-loader "parser" '() 'QUERY)))
-(add-subsystem-identification! "*Parser" '(0 12))
\ No newline at end of file
+    (load-package-set "parser")))
+(add-subsystem-identification! "*Parser" '(0 13))
\ No newline at end of file
