@@ -1,10 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: xml-parser.scm,v 1.73 2007/01/17 03:43:09 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -68,12 +67,12 @@ USA.
 	  (set-cdr! elements (cddr elements)))))
   elements)
 
-(define (string-parser description alphabet)
-  (let ((a1 (alphabet- alphabet (string->alphabet "\"")))
-	(a2 (alphabet- alphabet (string->alphabet "'"))))
+(define (string-parser description char-set)
+  (let ((a1 (char-set-difference char-set (string->char-set "\"")))
+	(a2 (char-set-difference char-set (string->char-set "'"))))
     (*parser
-     (alt (sbracket description "\"" "\"" (match (* (alphabet a1))))
-	  (sbracket description "'" "'" (match (* (alphabet a2))))))))
+     (alt (sbracket description "\"" "\"" (match (* (char-set a1))))
+	  (sbracket description "'" "'" (match (* (char-set a2))))))))
 
 ;;;; Entry points
 
@@ -83,8 +82,8 @@ USA.
       (read-xml port (if (default-object? pi-handlers) '() pi-handlers)))))
 
 (define (read-xml port #!optional pi-handlers)
-  (let ((coding (determine-coding port)))
-    (parse-xml (input-port->parser-buffer port)
+  (receive (coding prefix) (determine-coding port)
+    (parse-xml (input-port->parser-buffer port prefix)
 	       coding
 	       (guarantee-pi-handlers pi-handlers 'READ-XML))))
 
@@ -118,83 +117,88 @@ USA.
 ;;;; Character coding
 
 (define (determine-coding port)
-  (port/set-coding port 'ISO-8859-1)
-  (port/set-line-ending port 'XML-1.0)
-  (receive (coding name char) (determine-coding-1 port)
-    (if coding (port/set-coding port coding))
-    (if char (unread-char char port))
-    name))
+  (if (port/supports-coding? port)
+      (begin
+	(port/set-coding port 'BINARY)
+	(port/set-line-ending port 'BINARY)
+	(receive (coding name prefix) (determine-coding-1 port)
+	  (port/set-coding port coding)
+	  (port/set-line-ending port 'XML-1.0)
+	  (values name prefix)))
+      (values #f #f)))
 
 (define (determine-coding-1 port)
-  (let ((rc
+  (let ((rb
 	 (lambda ()
 	   (let ((c (read-char port)))
 	     (if (eof-object? c)
 		 (error "EOF while determining char coding."))
-	     c)))
+	     (char->integer c))))
+	(prefix
+	 (lambda (n)
+	   (wide-string (integer->char n))))
 	(lose
-	 (lambda chars
-	   (error "Illegal starting bytes:" (map char->integer chars)))))
-    (let ((c0 (rc)))
-      (case c0
-	((#\U+00)
-	 (let* ((c1 (rc))
-		(c2 (rc))
-		(c3 (rc)))
-	   (if (and (char=? c1 #\U+00)
-		    (char=? c2 #\U+FE)
-		    (char=? c3 #\U+FF))
-	       (values 'UTF-32BE 'UTF-32 #f)
-	       (lose c0 c1 c2 c3))))
-	((#\U+EF)
-	 (let* ((c1 (rc))
-		(c2 (rc)))
-	   (if (and (char=? c1 #\U+BB) (char=? c2 #\U+BF))
-	       (values 'UTF-8 'UTF-8 #f)
-	       (lose c0 c1 c2))))
-	((#\U+FE)
-	 (let ((c1 (rc)))
-	   (if (char=? c1 #\U+FF)
-	       (values 'UTF-16BE 'UTF-16 #f)
-	       (lose c0 c1))))
-	((#\U+FF)
-	 (let* ((c1 (rc))
-		(c2 (rc))
-		(c3 (rc)))
-	   (if (char=? c1 #\U+FE)
-	       (if (and (char=? c2 #\U+00) (char=? c3 #\U+00))
-		   (values 'UTF-32LE 'UTF-32 #f)
-		   (values 'UTF-16LE
-			   'UTF-16
-			   (wide-string-ref
-			    (utf16-le-string->wide-string (string c2 c3))
-			    0)))
-	       (lose c0 c1 c2 c3))))
-	((#\U+3C)
-	 (values #f 'NO-BOM #\<))
+	 (lambda bytes
+	   (error "Illegal starting bytes:" bytes))))
+    (let ((b0 (rb)))
+      (case b0
+	((#x00)
+	 (let* ((b1 (rb))
+		(b2 (rb))
+		(b3 (rb)))
+	   (if (not (and (fix:= b1 #x00)
+			 (fix:= b2 #xFE)
+			 (fix:= b3 #xFF)))
+	       (lose b0 b1 b2 b3)))
+	 (values 'UTF-32BE 'UTF-32 #f))
+	((#xEF)
+	 (let* ((b1 (rb))
+		(b2 (rb)))
+	   (if (not (and (fix:= b1 #xBB)
+			 (fix:= b2 #xBF)))
+	       (lose b0 b1 b2)))
+	 (values 'UTF-8 'UTF-8 #f))
+	((#xFE)
+	 (let ((b1 (rb)))
+	   (if (not (fix:= b1 #xFF))
+	       (lose b0 b1)))
+	 (values 'UTF-16BE 'UTF-16 #f))
+	((#xFF)
+	 (let* ((b1 (rb))
+		(b2 (rb))
+		(b3 (rb)))
+	   (if (not (fix:= b1 #xFE))
+	       (lose b0 b1 b2 b3))
+	   (if (and (fix:= b2 #x00) (fix:= b3 #x00))
+	       (values 'UTF-32LE 'UTF-32 #f)
+	       (values 'UTF-16LE 'UTF-16
+		       (prefix (fix:or (fix:lsh b3 8) b2))))))
 	(else
-	 (values 'UTF-8 'UTF-8 c0))))))
+	 (values 'UTF-8
+		 (if (fix:= b0 #x3C) 'NO-BOM 'UTF-8)
+		 (prefix b0)))))))
 
 (define (finish-coding buffer coding declaration)
-  (let ((port (parser-buffer-port buffer)))
-    (if port
-	(let* ((declared (normalize-coding port declaration))
-	       (lose
-		(lambda ()
-		  (error "Incorrect encoding declaration:" declared))))
-	  (case coding
-	    ((UTF-8 UTF-16)
-	     (if (not (or (not declared) (eq? declared coding)))
-		 (lose)))
-	    ((UTF-32)
-	     (if (not (eq? declared coding))
-		 (lose)))
-	    ((NO-BOM)
-	     (if (coding-requires-bom? declared)
-		 (lose))
-	     (port/set-coding port (or declared 'UTF-8)))
-	    ((ANY) unspecific)
-	    (else (error:bad-range-argument coding #f)))))))
+  (if coding
+      (let ((port (parser-buffer-port buffer)))
+	(if port
+	    (let* ((declared (normalize-coding port declaration))
+		   (lose
+		    (lambda ()
+		      (error "Incorrect encoding declaration:" declared))))
+	      (case coding
+		((UTF-8 UTF-16)
+		 (if (not (or (not declared) (eq? declared coding)))
+		     (lose)))
+		((UTF-32)
+		 (if (not (eq? declared coding))
+		     (lose)))
+		((NO-BOM)
+		 (if (coding-requires-bom? declared)
+		     (lose))
+		 (port/set-coding port (or declared 'UTF-8)))
+		((ANY) unspecific)
+		(else (error:bad-range-argument coding #f))))))))
 
 (define (normalize-coding port declaration)
   (let ((coding
@@ -203,8 +207,7 @@ USA.
 		(and coding
 		     (intern coding))))))
     (if (and coding
-	     (not (port/known-coding? port coding))
-	     (port/supports-coding? port))
+	     (not (port/known-coding? port coding)))
 	(error:bad-range-argument coding #f))
     coding))
 
@@ -379,7 +382,14 @@ USA.
 		(set-xml-attribute-name! attr
 					 (expand-attribute-name
 					  (xml-attribute-name attr))))
-	      attrs)))
+	      attrs)
+    (do ((attrs attrs (cdr attrs)))
+	((not (pair? attrs)) unspecific)
+      (let ((name (xml-attribute-name (car attrs))))
+	(if (there-exists? (cdr attrs)
+	      (lambda (attr)
+		(xml-name=? (xml-attribute-name attr) name)))
+	    (perror p "Attributes with same name" (xml-name->symbol name)))))))
 
 (define (parse-element-content b p name)
   (let ((vc (parse-content b)))
@@ -390,7 +400,8 @@ USA.
 	  (if (peek-parser-buffer-char b)
 	      (perror (get-parser-buffer-pointer b) "Unknown content")
 	      (perror p "Unterminated start tag" name)))
-      (if (not (xml-name=? (vector-ref ve 0) name))
+      (if (not (eq? (xml-name->symbol (vector-ref ve 0))
+		    (xml-name->symbol name)))
 	  (perror p "Mismatched start tag" (vector-ref ve 0) name))
       (let ((content (coalesce-strings! (vector->list vc))))
 	(if (null? content)
@@ -419,12 +430,12 @@ USA.
 
 ;;;; Attribute defaulting
 
-(define (process-attr-decls qname attrs p)
+(define (process-attr-decls name attrs p)
   (let ((decl
 	 (and (or *standalone?* *internal-dtd?*)
 	      (find-matching-item *att-decls*
 		(lambda (decl)
-		  (xml-name=? (xml-!attlist-name decl) qname))))))
+		  (xml-name=? (xml-!attlist-name decl) name))))))
     (if decl
 	(do ((defns (xml-!attlist-definitions decl) (cdr defns))
 	     (attrs attrs (process-attr-defn (car defns) attrs p)))
@@ -432,42 +443,42 @@ USA.
 	attrs)))
 
 (define (process-attr-defn defn attrs p)
-  (let ((qname (car defn))
+  (let ((name (car defn))
 	(type (cadr defn))
 	(default (caddr defn)))
     (let ((attr
 	   (find-matching-item attrs
 	     (lambda (attr)
-	       (xml-name=? (car (xml-attribute-name attr)) qname)))))
+	       (xml-name=? (car (xml-attribute-name attr)) name)))))
       (if attr
 	  (let ((av (xml-attribute-value attr)))
 	    (if (and (pair? default)
 		     (eq? (car default) '|#FIXED|)
 		     (not (string=? av (cdr default))))
-		(perror (cdar attr) "Incorrect attribute value" qname))
+		(perror (cdar attr) "Incorrect attribute value" name))
 	    (if (not (eq? type '|CDATA|))
 		(set-xml-attribute-value! attr (trim-attribute-whitespace av)))
 	    attrs)
 	  (begin
 	    (if (eq? default '|#REQUIRED|)
-		(perror p "Missing required attribute value" qname))
+		(perror p "Missing required attribute value" name))
 	    (if (pair? default)
-		(cons (%make-xml-attribute (cons qname p) (cdr default)) attrs)
+		(cons (%make-xml-attribute (cons name p) (cdr default)) attrs)
 		attrs))))))
 
 ;;;; Other markup
 
 (define (bracketed-region-parser description start end)
   (let ((parser
-	 (terminated-region-parser description alphabet:xml-char #t end)))
+	 (terminated-region-parser description char-set:xml-char #t end)))
     (*parser (sbracket description start end parser))))
 
-(define (terminated-region-parser description alphabet must-match? end)
+(define (terminated-region-parser description char-set must-match? end)
   (let ((matcher
-	 (terminated-region-matcher description alphabet must-match? end)))
+	 (terminated-region-matcher description char-set must-match? end)))
     (*parser (map normalize-line-endings (match matcher)))))
 
-(define (terminated-region-matcher description alphabet must-match? . ends)
+(define (terminated-region-matcher description char-set must-match? . ends)
   description
   (lambda (buffer)
     (let loop ()
@@ -475,7 +486,7 @@ USA.
 	       (lambda (end)
 		 (match-parser-buffer-string-no-advance buffer end)))
 	     #t)
-	    ((match-parser-buffer-char-in-alphabet buffer alphabet)
+	    ((match-parser-buffer-char-in-set buffer char-set)
 	     (loop))
 	    (must-match?
 	     (let ((p (get-parser-buffer-pointer buffer))
@@ -488,7 +499,7 @@ USA.
 (define parse-char-data			;[14]
   (let ((parse-body
 	 (terminated-region-parser "character data"
-				   alphabet:char-data
+				   char-set:char-data
 				   #f
 				   "]]>")))
     (*parser
@@ -500,7 +511,7 @@ USA.
 
 (define parse-comment			;[15]
   (let ((parse-body
-	 (terminated-region-parser "comment" alphabet:xml-char #t "--")))
+	 (terminated-region-parser "comment" char-set:xml-char #t "--")))
     (*parser
      (encapsulate
 	 (lambda (v)
@@ -525,37 +536,21 @@ USA.
 (define parse-unexpanded-name		;[5]
   (*parser
    (with-pointer p
-     (map (lambda (s) (cons (make-xml-qname s) p))
-	  (match match-name)))))
+     (map (lambda (s) (cons (make-xml-name s) p))
+	  (match match:xml-name)))))
 
 (define (simple-name-parser type)
   (let ((m (string-append "Malformed " type " name")))
-    (*parser (require-success m (map make-xml-qname (match match-name))))))
+    (*parser (require-success m (map make-xml-name (match match:xml-name))))))
 
 (define parse-entity-name (simple-name-parser "entity"))
 (define parse-pi-name (simple-name-parser "processing-instructions"))
 (define parse-notation-name (simple-name-parser "notation"))
 
-(define (match-name buffer)
-  (and (match-parser-buffer-char-in-alphabet buffer alphabet:name-initial)
-       (let loop ()
-	 (if (match-parser-buffer-char-in-alphabet buffer
-						   alphabet:name-subsequent)
-	     (loop)
-	     #t))))
-
-(define parse-required-name-token	;[7]
+(define parse-required-nmtoken		;[7]
   (*parser
    (require-success "Malformed XML name token"
-     (map make-xml-nmtoken (match match-name-token)))))
-
-(define (match-name-token buffer)
-  (and (match-parser-buffer-char-in-alphabet buffer alphabet:name-subsequent)
-       (let loop ()
-	 (if (match-parser-buffer-char-in-alphabet buffer
-						   alphabet:name-subsequent)
-	     (loop)
-	     #t))))
+     (map make-xml-nmtoken (match match:xml-nmtoken)))))
 
 ;;;; Namespaces
 
@@ -566,36 +561,31 @@ USA.
 	      (let ((uname (xml-attribute-name (car attrs)))
 		    (value (xml-attribute-value (car attrs)))
 		    (tail (loop (cdr attrs))))
-		(let ((qname (car uname))
+		(let ((name (car uname))
 		      (p (cdr uname)))
-		  (let ((get-uri
+		  (let ((forbidden-uri
 			 (lambda ()
-			   (if (string-null? value)
-			       (null-xml-namespace-uri)
-			       (string->absolute-uri value))))
-			(forbidden-uri
-			 (lambda (uri)
-			   (perror p "Forbidden namespace URI"
-				   (uri->string uri)))))
-		    (let ((guarantee-legal-uri
-			   (lambda (uri)
-			     (if (or (uri=? uri xml-uri)
-				     (uri=? uri xmlns-uri))
-				 (forbidden-uri uri)))))
-		      (cond ((xml-name=? qname 'xmlns)
-			     (let ((uri (get-uri)))
-			       (guarantee-legal-uri uri)
-			       (cons (cons (null-xml-name-prefix) uri) tail)))
-			    ((xml-name-prefix=? qname 'xmlns)
-			     (if (xml-name=? qname 'xmlns:xmlns)
-				 (perror p "Illegal namespace prefix" qname))
-			     (let ((uri (get-uri)))
-			       (if (xml-name=? qname 'xmlns:xml)
-				   (if (not (uri=? uri xml-uri))
-				       (forbidden-uri uri))
-				   (guarantee-legal-uri uri))
-			       (cons (cons (xml-name-local qname) uri) tail)))
-			    (else tail))))))
+			   (perror p "Forbidden namespace URI" value))))
+		    (cond ((xml-name=? name 'xmlns)
+			   (let ((uri (string->absolute-uri value)))
+			     (if (or (uri=? value xml-uri)
+				     (uri=? value xmlns-uri))
+				 (forbidden-uri))
+			     (cons (cons (null-xml-name-prefix) uri)
+				   tail)))
+			  ((and (xml-qname? name)
+				(xml-name-prefix=? name 'xmlns))
+			   (if (xml-name=? name 'xmlns:xmlns)
+			       (perror p "Illegal namespace prefix" name))
+			   (string->uri value) ;signals error if not URI
+			   (if (if (xml-name=? name 'xmlns:xml)
+				   (not (string=? value xml-uri-string))
+				   (or (string-null? value)
+				       (string=? value xml-uri-string)
+				       (string=? value xmlns-uri-string)))
+			       (forbidden-uri))
+			   (cons (cons (xml-name-local name) value) tail))
+			  (else tail)))))
 	      *prefix-bindings*)))
   unspecific)
 
@@ -603,14 +593,15 @@ USA.
 (define (expand-attribute-name uname) (expand-name uname #t))
 
 (define (expand-name uname attribute-name?)
-  (let ((qname (car uname))
+  (let ((name (car uname))
 	(p (cdr uname)))
-    (if *in-dtd?*
-	qname
-	(let ((uri (lookup-namespace-prefix qname p attribute-name?)))
+    (if (or *in-dtd?*
+	    (not (xml-qname? name)))
+	name
+	(let ((uri (lookup-namespace-prefix name p attribute-name?)))
 	  (if (null-xml-namespace-uri? uri)
-	      qname
-	      (%make-xml-name qname uri))))))
+	      name
+	      (%make-xml-name name uri))))))
 
 (define (lookup-namespace-prefix qname p attribute-name?)
   (let ((prefix (xml-qname-prefix qname)))
@@ -637,7 +628,7 @@ USA.
 	(start "<?")
 	(end "?>"))
     (let ((parse-body
-	   (terminated-region-parser description alphabet:xml-char #t end)))
+	   (terminated-region-parser description char-set:xml-char #t end)))
       (*parser
        (with-pointer p
 	 (transform
@@ -683,14 +674,13 @@ USA.
   (let ((make-ref
 	 (lambda (s r p)
 	   (let ((n (string->number s r)))
-	     (if (not (unicode-code-point? n))
+	     (if (not (unicode-scalar-value? n))
 		 (perror p "Invalid code point" n))
 	     (let ((char (integer->char n)))
-	       (if (not (char-in-alphabet? char alphabet:xml-char))
+	       (if (not (char-set-member? char-set:xml-char char))
 		   (perror p "Disallowed Unicode character" char))
-	       (call-with-output-string
+	       (call-with-utf8-output-string
 		 (lambda (port)
-		   (port/set-coding port 'UTF-8)
 		   (write-char char port))))))))
     (*parser
      (with-pointer p
@@ -723,7 +713,7 @@ USA.
 	 (alt (seq "#"
 		   (alt match-decimal
 			(seq "x" match-hexadecimal)))
-	      match-name)
+	      match:xml-name)
 	 ";"))))
 
 (define parse-entity-reference-name	;[68]
@@ -732,7 +722,7 @@ USA.
      parse-entity-name)))
 
 (define parse-entity-reference-deferred
-  (*parser (match (seq "&" match-name ";"))))
+  (*parser (match (seq "&" match:xml-name ";"))))
 
 (define parse-parameter-entity-reference-name ;[69]
   (*parser
@@ -782,12 +772,12 @@ USA.
 			 (lambda (a) (car a))))
 
 (define parse-declaration-attributes
-  (attribute-list-parser (*parser (map make-xml-qname (match match-name)))
+  (attribute-list-parser (*parser (map make-xml-name (match match:xml-name)))
 			 (lambda (a) a)))
 
-(define (attribute-value-parser alphabet parse-reference)
-  (let ((a1 (alphabet- alphabet (string->alphabet "\"")))
-	(a2 (alphabet- alphabet (string->alphabet "'"))))
+(define (attribute-value-parser char-set parse-reference)
+  (let ((a1 (char-set-difference char-set (string->char-set "\"")))
+	(a2 (char-set-difference char-set (string->char-set "'"))))
     (*parser
      (encapsulate (lambda (v)
 		    (let ((elements (coalesce-strings! (vector->list v))))
@@ -795,15 +785,15 @@ USA.
 			  (list "")
 			  elements)))
        (alt (sbracket "attribute value" "\"" "\""
-	      (* (alt (match (+ (alphabet a1)))
+	      (* (alt (match (+ (char-set a1)))
 		      parse-reference)))
 	    (sbracket "attribute value" "'" "'"
-	      (* (alt (match (+ (alphabet a2)))
+	      (* (alt (match (+ (char-set a2)))
 		      parse-reference))))))))
 
 (define parse-entity-value		;[9]
   (attribute-value-parser
-   (alphabet- alphabet:xml-char (string->alphabet "%&"))
+   (char-set-difference char-set:xml-char (string->char-set "%&"))
    (*parser
     (alt parse-char-reference
 	 parse-entity-reference-deferred
@@ -821,7 +811,7 @@ USA.
 
 (define parse-attribute-value		;[10]
   (let ((parser
-	 (attribute-value-parser alphabet:char-data
+	 (attribute-value-parser char-set:char-data
 				 parse-reference-deferred)))
     (*parser
      (map (lambda (elements)
@@ -835,7 +825,7 @@ USA.
 ;;;; Normalization
 
 (define (normalize-attribute-value string)
-  (call-with-output-string
+  (call-with-utf8-output-string
     (lambda (port)
       (let normalize-string ((string string))
 	(let ((b (utf8-string->parser-buffer (normalize-line-endings string))))
@@ -869,7 +859,7 @@ USA.
 		 (loop))))))))))
 
 (define (trim-attribute-whitespace string)
-  (call-with-output-string
+  (call-with-utf8-output-string
    (lambda (port)
      (let ((string (string-trim string)))
        (let ((end (string-length string)))
@@ -1139,8 +1129,8 @@ USA.
 	 parse-required-element-name
 	 S
 	 ;;[46]
-	 (alt (map make-xml-qname (match "EMPTY"))
-	      (map make-xml-qname (match "ANY"))
+	 (alt (map make-xml-name (match "EMPTY"))
+	      (map make-xml-name (match "ANY"))
 	      ;;[51]
 	      (encapsulate vector->list
 		(with-pointer p
@@ -1190,14 +1180,14 @@ USA.
 
 (define parse-!attlist-type		;[54,57]
   (*parser
-   (alt (map make-xml-qname
+   (alt (map make-xml-name
 	     ;;[55,56]
 	     (match (alt "CDATA" "IDREFS" "IDREF" "ID"
 			 "ENTITY" "ENTITIES" "NMTOKENS" "NMTOKEN")))
 	;;[58]
 	(encapsulate vector->list
 	  (bracket "notation type"
-	      (seq (map make-xml-qname (match "NOTATION"))
+	      (seq (map make-xml-name (match "NOTATION"))
 		   S
 		   "(")
 	      ")"
@@ -1209,8 +1199,8 @@ USA.
 	(encapsulate (lambda (v) (cons 'enumerated (vector->list v)))
 	  (sbracket "enumerated type" "(" ")"
 	    S?
-	    parse-required-name-token
-	    (* (seq S? "|" S? parse-required-name-token))
+	    parse-required-nmtoken
+	    (* (seq S? "|" S? parse-required-nmtoken))
 	    S?)))))
 
 (define parse-!attlist-default		;[60]
@@ -1286,14 +1276,13 @@ USA.
 		 parse-system-literal))))))
 
 (define parse-system-literal		;[11]
-  (string-parser "system literal" alphabet:xml-char))
+  (string-parser "system literal" char-set:xml-char))
 
 (define parse-public-id-literal		;[12,13]
   (string-parser "public-ID literal"
-		 (char-set->alphabet
-		  (char-set-union
-		   char-set:alphanumeric
-		   (string->char-set " \r\n-'()+,./:=?;!*#@$_%")))))
+		 (char-set-union
+		  char-set:alphanumeric
+		  (string->char-set " \r\n-'()+,./:=?;!*#@$_%"))))
 
 ;;;; External subset
 
@@ -1310,9 +1299,9 @@ USA.
 	   parse-decl-separator))))
 
 (define external-decl-parser
-  (let ((a1 (alphabet- alphabet:xml-char (string->alphabet "%\"'>")))
-	(a2 (alphabet- alphabet:xml-char (string->alphabet "\"")))
-	(a3 (alphabet- alphabet:xml-char (string->alphabet "'"))))
+  (let ((a1 (char-set-difference char-set:xml-char (string->char-set "%\"'>")))
+	(a2 (char-set-difference char-set:xml-char (string->char-set "\"")))
+	(a3 (char-set-difference char-set:xml-char (string->char-set "'"))))
     (lambda (prefix parse-decl)
       (*parser
        (with-pointer p
@@ -1324,9 +1313,9 @@ USA.
 	    (require-success "Malformed markup declaration"
 	      (seq
 	       (* (alt (match
-			(alt (* (alphabet a1))
-			     (seq (char #\") (* (alphabet a2)) (char #\"))
-			     (seq (char #\') (* (alphabet a3)) (char #\'))))
+			(alt (+ (char-set a1))
+			     (seq (char #\") (* (char-set a2)) (char #\"))
+			     (seq (char #\') (* (char-set a3)) (char #\'))))
 		       parse-parameter-entity-reference))
 	       (match ">"))))))))))
 
@@ -1406,7 +1395,7 @@ USA.
 		match-!ignore)))))
 
 (define match-!ignore			;[65]
-  (terminated-region-matcher "ignore section" alphabet:xml-char #t
+  (terminated-region-matcher "ignore section" char-set:xml-char #t
 			     conditional-start conditional-end))
 
 (define parse-parameterized-conditional
