@@ -1,10 +1,9 @@
 #| -*-Scheme-*-
 
-$Id: xml-struct.scm,v 1.59 2008/01/30 20:02:42 cph Exp $
-
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -123,6 +122,16 @@ USA.
 (define (xml-whitespace-string? object)
   (string-composed-of? object char-set:xml-whitespace))
 
+(define (string-composed-of? string char-set)
+  (and (string? string)
+       (substring-composed-of? string 0 (string-length string) char-set)))
+
+(define (substring-composed-of? string start end char-set)
+  (let loop ((index start))
+    (or (fix:= index end)
+	(and (char-set-member? char-set (string-ref string index))
+	     (loop (fix:+ index 1))))))
+
 (define-xml-type declaration
   (version xml-version?)
   (encoding xml-encoding?)
@@ -154,24 +163,23 @@ USA.
   (value canonicalize canonicalize-char-data))
 
 (define (xml-char-data? object)
-  (or (wide-char? object)
+  (or (unicode-char? object)
       (and (or (wide-string? object)
                (and (string? object)
                     (utf8-string-valid? object)))
            (string-of-xml-chars? object))))
 
 (define (string-of-xml-chars? string)
-  (for-all-chars-in-string? (alphabet-predicate alphabet:xml-char)
+  (for-all-chars-in-string? (char-set-predicate char-set:xml-char)
                             string
                             0
                             (string-length string)
                             'UTF-8))
 
 (define (canonicalize-char-data object)
-  (cond ((wide-char? object)
-	 (call-with-output-string
+  (cond ((unicode-char? object)
+	 (call-with-utf8-output-string
 	   (lambda (port)
-	     (port/set-coding port 'UTF-8)
 	     (write-char object port))))
 	((wide-string? object)
 	 (wide-string->utf8-string object))
@@ -201,6 +209,8 @@ USA.
 		  (loop (cdr attrs)))
 	     #t))))
 
+(define-guarantee xml-attribute-list "XML attribute list")
+
 (define (xml-content? object)
   (list-of-type? object xml-content-item?))
 
@@ -242,6 +252,46 @@ USA.
 		       (xml-processing-instructions? item)))
 	      (error:wrong-type-datum content "an XML content")))))
     (search content)))
+
+(define (xml-element-child name elt #!optional error?)
+  (let ((child
+	 (let ((name (xml-name-arg name 'XML-ELEMENT-CHILD)))
+	   (find (lambda (item)
+		   (and (xml-element? item)
+			(xml-name=? (xml-element-name item) name)))
+		 (xml-element-content elt)))))
+    (if (and (not child) (if (default-object? error?) #f error?))
+	(error:bad-range-argument name 'XML-ELEMENT-CHILD))
+    child))
+
+(define (xml-element-children name elt)
+  (let ((name (xml-name-arg name 'XML-ELEMENT-CHILDREN)))
+    (filter (lambda (item)
+	      (and (xml-element? item)
+		   (xml-name=? (xml-element-name item) name)))
+	    (xml-element-content elt))))
+
+(define (find-xml-attr name elt #!optional error?)
+  (let ((attr
+	 (find (let ((name (xml-name-arg name 'FIND-XML-ATTR)))
+		 (lambda (attr)
+		   (xml-name=? (xml-attribute-name attr) name)))
+	       (if (xml-element? elt)
+		   (xml-element-attributes elt)
+		   (begin
+		     (guarantee-xml-attribute-list elt 'FIND-XML-ATTR)
+		     elt)))))
+    (if (and (not attr) (if (default-object? error?) #f error?))
+	(error:bad-range-argument name 'FIND-XML-ATTR))
+    (and attr
+	 (xml-attribute-value attr))))
+
+(define (xml-name-arg arg caller)
+  (if (string? arg)
+      (make-xml-name arg)
+      (begin
+	(guarantee-xml-name arg caller)
+	arg)))
 
 (define-xml-type comment
   (text canonicalize canonicalize-char-data))
@@ -249,12 +299,12 @@ USA.
 (define-xml-type processing-instructions
   (name
    (lambda (object)
-     (and (xml-qname? object)
+     (and (xml-name-symbol? object)
 	  (not (xml-name=? object 'xml)))))
   (text canonicalize canonicalize-char-data))
 
 (define-xml-type dtd
-  (root xml-qname?)
+  (root xml-name-symbol?)
   (external (lambda (object)
 	      (or (not object)
 		  (xml-external-id? object))))
@@ -287,14 +337,14 @@ USA.
 		  (string->char-set " \r\n-'()+,./:=?;!*#@$_%")))
 
 (define-xml-type !element
-  (name xml-qname?)
+  (name xml-name-symbol?)
   (content-type
    (lambda (object)
      (or (eq? object '|EMPTY|)
 	 (eq? object '|ANY|)
 	 (and (pair? object)
 	      (eq? '|#PCDATA| (car object))
-	      (list-of-type? (cdr object) xml-qname?))
+	      (list-of-type? (cdr object) xml-name-symbol?))
 	 (letrec
 	     ((children?
 	       (lambda (object)
@@ -306,7 +356,7 @@ USA.
 			  (list-of-type? (cdr object) cp?))))))
 	      (cp?
 	       (lambda (object)
-		 (or (maybe-wrapped object xml-qname?)
+		 (or (maybe-wrapped object xml-name-symbol?)
 		     (children? object))))
 	      (maybe-wrapped
 	       (lambda (object pred)
@@ -321,13 +371,13 @@ USA.
 	   (children? object))))))
 
 (define-xml-type !attlist
-  (name xml-qname?)
+  (name xml-name-symbol?)
   (definitions canonicalize
     (lambda (object)
       (if (not (list-of-type? object
 		 (lambda (item)
 		   (and (pair? item)
-			(xml-qname? (car item))
+			(xml-name-symbol? (car item))
 			(pair? (cdr item))
 			(!attlist-type? (cadr item))
 			(pair? (cddr item))
@@ -354,7 +404,7 @@ USA.
       (eq? object '|NMTOKEN|)
       (and (pair? object)
 	   (or (and (eq? (car object) '|NOTATION|)
-		    (list-of-type? (cdr object) xml-qname?))
+		    (list-of-type? (cdr object) xml-name-symbol?))
 	       (and (eq? (car object) 'enumerated)
 		    (list-of-type? (cdr object) xml-nmtoken?))))))
 
@@ -367,16 +417,16 @@ USA.
 	   (xml-char-data? (cdr object)))))
 
 (define-xml-type !entity
-  (name xml-qname?)
+  (name xml-name-symbol?)
   (value canonicalize canonicalize-entity-value))
 
 (define-xml-type unparsed-!entity
-  (name xml-qname?)
+  (name xml-name-symbol?)
   (id xml-external-id?)
-  (notation xml-qname?))
+  (notation xml-name-symbol?))
 
 (define-xml-type parameter-!entity
-  (name xml-qname?)
+  (name xml-name-symbol?)
   (value canonicalize canonicalize-entity-value))
 
 (define (canonicalize-entity-value object)
@@ -393,14 +443,14 @@ USA.
 	(canonicalize-content object))))
 
 (define-xml-type !notation
-  (name xml-qname?)
+  (name xml-name-symbol?)
   (id xml-external-id?))
 
 (define-xml-type entity-ref
-  (name xml-qname?))
+  (name xml-name-symbol?))
 
 (define-xml-type parameter-entity-ref
-  (name xml-qname?))
+  (name xml-name-symbol?))
 
 (define-syntax define-xml-printer
   (sc-macro-transformer
@@ -429,7 +479,7 @@ USA.
 
 (define-xml-printer element
   (lambda (elt)
-    (xml-name-qname (xml-element-name elt))))
+    (xml-name->symbol (xml-element-name elt))))
 
 (define-xml-printer external-id
   (lambda (dtd)
@@ -442,8 +492,8 @@ USA.
 	(xml-name-prefix=? name 'xmlns))))
 
 (define (xml-element-namespace-decls elt)
-  (keep-matching-items (xml-element-attributes elt)
-    xml-attribute-namespace-decl?))
+  (filter xml-attribute-namespace-decl?
+	  (xml-element-attributes elt)))
 
 (define (xml-element-namespace-uri elt prefix)
   (let ((value
@@ -458,10 +508,10 @@ USA.
 
 (define (xml-element-namespace-prefix elt uri-string)
   (let ((attr
-	 (find-matching-item (xml-element-attributes elt)
-	   (lambda (attr)
-	     (and (xml-attribute-namespace-decl? attr)
-		  (string=? (xml-attribute-value attr) uri-string))))))
+	 (find (lambda (attr)
+		 (and (xml-attribute-namespace-decl? attr)
+		      (string=? (xml-attribute-value attr) uri-string)))
+	       (xml-element-attributes elt))))
     (and attr
 	 (let ((name (xml-attribute-name attr)))
 	   (if (xml-name=? name 'xmlns)
@@ -485,7 +535,7 @@ USA.
 (define (xml-stylesheet . items)
   (make-xml-processing-instructions
    'xml-stylesheet
-   (call-with-output-string
+   (call-with-utf8-output-string
      (lambda (port)
        (for-each (lambda (attr)
 		   (write-char #\space port)
@@ -516,9 +566,9 @@ USA.
 (define (xml-attrs . items)
   (let ((flush
 	 (lambda (name attrs)
-	   (delete-matching-items! attrs
-	     (lambda (attr)
-	       (eq? (xml-attribute-name attr) name))))))
+	   (remove! (lambda (attr)
+		      (eq? (xml-attribute-name attr) name))
+		    attrs))))
     (let ((accum
 	   (lambda (attr attrs)
 	     (cons attr (flush (xml-attribute-name attr) attrs)))))
@@ -526,18 +576,23 @@ USA.
 	(if (pair? items)
 	    (let ((item (car items))
 		  (items (cdr items)))
-	      (cond ((and (xml-name? item)
+	      (cond ((and (or (xml-name? item)
+			      (string? item))
 			  (pair? items))
-		     (let ((value (car items))
+		     (let ((name
+			    (if (string? item)
+				(make-xml-name item)
+				item))
+			   (value (car items))
 			   (attrs (loop (cdr items))))
 		       (if value
 			   (accum (make-xml-attribute
-				   item
+				   name
 				   (if (eq? value #t)
-				       (symbol-name item)
+				       (xml-name-string name)
 				       (convert-xml-string-value value)))
 				  attrs)
-			   (flush item attrs))))
+			   (flush name attrs))))
 		    ((xml-attribute? item)
 		     (accum item (loop items)))
 		    ((list-of-type? item xml-attribute?)
@@ -547,21 +602,6 @@ USA.
 		    (else
 		     (error "Unknown item passed to xml-attrs:" item))))
 	    '())))))
-
-(define (find-xml-attr name elt)
-  (guarantee-xml-name name 'FIND-XML-ATTR)
-  (let loop
-      ((attrs
-	(if (xml-element? elt)
-	    (xml-element-attributes elt)
-	    (begin
-	      (guarantee-list-of-type elt xml-attribute? "XML attributes"
-				      'FIND-XML-ATTR)
-	      elt))))
-    (and (pair? attrs)
-	 (if (xml-name=? (xml-attribute-name (car attrs)) name)
-	     (xml-attribute-value (car attrs))
-	     (loop (cdr attrs))))))
 
 (define (flatten-xml-element-content item)
   (letrec
